/**
 *
 * Fri Mar 06 00:00:00 1998 AnnotationRenderer.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Observer;

public class AnnotationRenderer extends DataRenderer
{
	private DataFilter filter;
	private Vector list = new Vector();
	private double[] original_coords;
	protected Color color = Color.black;
	
	public void SetColor( Color c )
	{ if (c!=null) color = c; }
	
	public Color color() 
	{	return color; }	
	
	public void SetCoordFilter( DataFilter d )
	{ filter = d; filter.SetInput(this); SetInput(d); }
	
	private synchronized void buildCoords()
	// use our mystery datafilter to translate coordinates for us
	{
		if (filter==null) return;
		setChanged(); notifyObservers();
	}
	
	public void draw( Graphics g )
	{
		double[] coords = filter.getData();
		if (coords==null || coords.length<2) return;
		
		int x,y;
		g.setColor( color );
		for( int i=0; i<list.size(); i++ ) {
			x = (int)(coords[i<<1]);
			y = (int)(coords[(i<<1)+1]);
			g.drawString( ((Annotation)list.elementAt(i)).text, x,y );
			System.out.println("painted at "+x+","+y);
		}
	}
	
	public synchronized void AddAnnotation( Annotation x )
	{ list.addElement(x); setChanged(); notifyObservers(); }
	
	public synchronized int AnnotationCount()
	{ return list.size(); }
	
	public synchronized Annotation GetAnnotation( int i )
	{ return (Annotation)list.elementAt( i ); }
	
	public synchronized void RemoveAnnotation( Annotation i )
	{ list.removeElement( i ); setChanged(); notifyObservers();  }
	
		
	private synchronized void dirty()
	{
		original_coords=null;
		if (list.size()==0) return;
		original_coords = new double[ 2*list.size() ];
		for( int i=0; i<list.size(); i++ ) {
			original_coords[ i<<1 ] = ((Annotation)list.elementAt(i)).x;
			original_coords[ (i<<1)+1 ] = ((Annotation)list.elementAt(i)).y;
		}
		System.out.println("coords built\n");
	}
	
	public synchronized void SetInput( DataSource _in )
	{
		dirty();
		super.SetInput( _in );
	}
	
	public void update( Observable it, Object data )
	{
		dirty(); // don't pass on the signal, or we'll go to loopyland
		if (it!=(Observable)in) {
			setChanged(); notifyObservers(); 
		}
	} 
	
	public double[] getData()  // for passing to the filter
	{ return original_coords; }
	
	public double[] getExtents() // min,max per dimension
	{
		return null;
	}

	public int[] getDimensions() 
	{ int[] ret = { 2, list.size() };
	  return ret; }
	
}

