/**
 *
 * Fri Apr 24 01:00:00 1998 AxesOrderFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.util.*;


public class AxesOrderFilter extends DataFilter{
  public static int X=0,Y=1,Z=2;
  protected int []swap = {0,1,2}, realorder={0,1,2};
  private double[] cache=null,ext=null;
  private boolean caching=false;
  /**
    *  reorder wants the new order of axes.
    * example: to put the X data set on the Z axis, use reorder(_,_,X);
    * NOT reorder(Z,_,_)
    * Reason: this way you can ignore a data set if you wanted to,
    * ie. reorder(X,Y,X) would make a hill-like graph in 3d
    * for each point (X,Y,Z), a point (X,Y,X) is created.
    * [the other way, inserting a data set twice would couse problems
    * ie reorder(X,Y,X) would give you a 3 dim data set with the value
    * for each point (X,Y,Z) a reordered value of(Z,Y,/)]
    **/
  public synchronized void reorderAxes(int x, int y){reorderAxes(x,y,Z);}
  public synchronized void reorderAxes(int x, int y, int z){
    reloadExts();
    int [] neworder = {x,y,z};
    System.arraycopy(neworder,0,swap,0,3);
    cache=null;
    ext=null;
    setChanged();notifyObservers();
  }
  private boolean equal(int[]x,int[]y){
    if(x.length!=y.length)
      return false;
    for(int z=0; z<x.length;z++)
      if(x[z]!=y[z])
	return false;
    return true;
  }
  public double[] getData(){
    //System.out.println("DBG: AxesOrderFilter:getData");
    if(in==null)
      return null;
    if(equal(swap,realorder))
      return in.getData();
    if(cache!=null)
      return cache;
    double[] old=in.getData(), out=new double[old.length];
    int dim=in.getDimensions()[0];
    for(int z=0;z<out.length;z+=dim)
      for(int n=0;n<dim;n++)
	out[z+n]=old[z+swap[n]];
    if(cacheEnabled())
      cache=out;
    return out;
  }
  public double[] getExtents(){
    //System.out.println("DBG: AxesOrderFilter:getExtents");
    //if(equal(swap,realorder))
    //return in.getExtents();
    if(in==null)
      return null;
    if(ext==null){
      double[] old=in.getExtents();
      ext=new double[old.length];
      int dim=in.getDimensions()[0];
      for(int z=0;z<ext.length;z+=dim){
	ext[z]=old[swap[z/dim]*dim];
	ext[z+1]=old[swap[z/dim]*dim+1];
      }
    }
    return ext;
  }
  private synchronized void reloadExts(){
    if((in==null)||(ext==null))
      return;
    double[] old=in.getExtents();
    int dim=in.getDimensions()[0];
    for(int z=0;z<ext.length;z+=dim){
      old[swap[z/dim]*dim]=ext[z];
      old[swap[z/dim]*dim+1]=ext[z+1];
    }
  }
  public void update(Observable s, Object j){
    setChanged();notifyObservers();
  } 
}

class ExtentsAxesOrder extends AxesOrderFilter{
  public double[] reorderExtents(double[] n){
    if(n==null)
      return null;
    double[] out=new double[n.length];
    int dim=n.length/2;
    for(int z=0;z<out.length;z+=dim){
      out[z]=n[swap[z/dim]*dim];
      out[z+1]=n[swap[z/dim]*dim+1];
    }
    return out;
  }
}

class AxesOrderGroup extends AxesOrderFilter{
  private Vector in = new Vector();
  /**
    *  reorder wants the new order, NOT the place you want to put it.
    * example: to put the X data set on the Z axis, use reorder(_,_,X);
    * NOT reorder(Z,_,_)
    **/
  //public void reorderAxes(int x, int y){reorderAxes(x,y,Z);}
  public void reorderAxes(int x, int y, int z){
    super.reorderAxes(x,y,z);
    for (int n = 0; n<in.size(); n++)
      ((AxesOrderFilter)in.elementAt(n)).reorderAxes(x,y,z);
  }

  public void AddInput(AxesOrderFilter _in){ 
    if (_in != null) {
      if (! in.contains(_in)){
	in.addElement(_in);
	//_in.addObserver(this);
      }
    }
  }
  public void RemoveInput( AxesOrderFilter out ) 
    {
      if (in.contains(out)){
	in.removeElement(out);
	//out.deleteObserver(this);
      }
    }
  public double[]getExtents(){return null;}
  public double[]getData(){return null;}
}
