/**
 *
 * Mon Apr 20 01:00:00 1998 CanvasView.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
import java.util.Date;

class CanvasDataView extends DataSource
{
	int w=1, h=1;
	static int[] dims = { 2 };
	
	public double[] getData()
	{ return null; }
	
	public int[] getDimensions()
	{ return dims; }
	
	public double[] getExtents()
	{
		double[] r = new double[4];
		r[0] = 0;
		r[1] = (double)w;
		r[2] = (double)h;
		r[3] = 0;
                //System.out.println("visible "+r[0]+" "+r[1]+" "+r[2]+" "+r[3]);
		return r;
	}
	
	public void setWH( int _w, int _h )
    { 
      //if((_w!=w) || (_h!=h)){
	w = _w;
	h = _h;
	setChanged();
	notifyObservers(); 
	//}
    }
}
	


public class CanvasView extends Canvas implements Observer, Runnable, Cloneable
{
	private CanvasDataView cdv = new CanvasDataView();	
	private CursorSource cs = null; 
	private Vector bg = new Vector();
	private DataRenderer rendy = null;
  private Color bcol = Color.lightGray;
  private Thread rendThread = null;
  // private long time = 0;
  private CanvasView loop = this;
  //  private int[] clipdims = null;
  public Frame hostframe=null;
  protected void canvasreshape( int x, int y, int _w, int _h ){
    super.reshape(x,y,_w,_h);
  }
	public void reshape( int x, int y, int _w, int _h )
	{ super.reshape(x,y,_w,_h);
	cdv.setWH(_w,_h); 
          //System.out.println("w="+_w+" h="+_h); 
        }

	public DataSource dataView()
	{ return cdv; }
	
	public void SetRenderer( DataRenderer _r )
	{ 
		if (rendy!=null) rendy.deleteObserver( this );
		rendy = _r; 
		if (rendy!=null) rendy.addObserver( this );
	}
	
	public void AddBackgroundRenderer( DataRenderer _r )
	{ bg.addElement( _r ); }
	
	public void RemoveBackgroundRenderer( Object _r )
	{ bg.removeElement( _r ); }
	
	public synchronized void SetCursorOutput( CursorSource _cs )
	/** optionally: track cursor into a cursor source */
	{ cs = _cs; }
  private void setFrameCursor(int n){
    try{
      hostframe.setCursor(n);
    }catch(Exception e){}
  }
    
  public CursorSource getCursorSource(){ return cs;}
	
 	public boolean mouseMove(Event evt, int  x, int  y) 
	{ 
	  //deliverEvent(evt);
	  //System.out.println("DBG: m "+evt.modifiers+" next = "+evt.evt);
          	if (cs!=null) { cs.SetPosition( x,y, 0 ); return false; }
		else return super.mouseMove(evt,x,y); 
	}
 	public boolean mouseDown(Event evt, int  x, int  y) 
	{ 
	  //deliverEvent(evt);
	  //System.out.println("DBG: d "+evt.modifiers+" next = "+evt.evt);
  	  //System.out.println("DBG: Mouse down");
	  if (cs!=null) { cs.SetPosition( x,y,  (evt.modifiers/4) +1 ); return false; }
		else return super.mouseDown(evt,x,y); 
	}
 	public boolean mouseUp(Event evt, int  x, int  y) 
	{ 
	  //deliverEvent(evt);
	  //System.out.println("DBG: u "+evt.modifiers+" next = "+evt.evt);
  	  //System.out.println("DBG: Mouse up");
	  if (cs!=null) { cs.SetPosition( x,y, 0 ); return false; }
	  else return super.mouseUp(evt,x,y); 
	}
  
 	public boolean mouseDrag(Event evt, int  x, int  y) 
	{ 
	  //deliverEvent(evt);
	  //System.out.println("DBG: h"+evt.modifiers+" next = "+evt.evt);
        	if (cs!=null) {cs.SetPosition( x,y, (evt.modifiers/4) +1 ); return false; }
		else return super.mouseDrag(evt,x,y); 
	}
  private int currentCursor = Frame.CROSSHAIR_CURSOR;
  public void cursor(int n){
    currentCursor=n;
    setFrameCursor(n);      
  }
  public boolean mouseEnter(Event evt, int  x, int  y) 
    { 
      setFrameCursor(currentCursor);
      return true;
    }
  public boolean mouseExit(Event evt, int  x, int  y) 
    { 
      setFrameCursor(Frame.DEFAULT_CURSOR);      
      return true;
    }
  public void setBackgroundColor(Color c){
    if (c!= null)
      bcol = c;
  }
  public void background(Graphics g){
    setBackground(bcol);
    //g.fillRect(0,0,(int)cdv.getExtents()[1],(int)cdv.getExtents()[2]);
  }
  public void draw( Graphics g )
    { 
      //System.out.println("DBG: Canvas Painting");
      //setBackground(bcol);
      for( int i=0; i<bg.size(); i++ ) 
	( (DataRenderer)bg.elementAt(i) ).paint(g); 
      if (rendy!=null) rendy.paint(g); 
    }
  protected Graphics gt = null;
  public final  void paint( Graphics g){
    //System.out.println("DBG: Canvas::paint(Graphics) called");
    if (g!= null){
      try{
	Rectangle th = g.getClipRect(), cu = gt.getClipRect();
	if((th.x<=cu.x)&&(th.y<=cu.y)&&
	   (th.x+th.width>=cu.x+cu.width) &&(th.y+th.height>=cu.y+cu.height) ){	  
	  //System.out.println("DBG: old clip included in new.  restart painting");
	  Cancel();
	}
	if((th.x>cu.x)&&(th.y>cu.y)&&
	   (th.x+th.width<cu.x+cu.width) &&(th.y+th.height<cu.y+cu.height) ){	  
	  //System.out.println("DBG: new clip included in old.  no painting");
	  return;
	}
      }catch(Exception ex){}
      if (rendy!=null){
	//Cancel();
	try{
	  CanvasView newcv = (CanvasView)(this.clone());
	  newcv.gt = g.create();
	  rendThread = new Thread(newcv);
	} catch(CloneNotSupportedException cnse){
	  System.out.println("DBG: Whoops.  Can't clone this.");
	  gt = g.create();
	  rendThread = new Thread(this);
	}
	if (rendy.ThreadPaint){
	  rendThread.setPriority(8);
	  rendThread.start();
	  System.gc();
	}else
	  rendThread.run();
      }else
	rendThread.run();
    }
  }
  public final void run(){
    loop.gt = gt;
    try{
      background(gt);
      draw(gt);
    }catch (Exception npe){
      //System.out.println("DBG: Null thrown.  Painting Thread Cancelled.");
    }
    gt.finalize();
  }
  /*private boolean Alive(){
    try{
      gt.create();
    } catch (NullPointerException npe){
      return false;
    }
    return true;
  }*/
  public void Cancel(){
    try{
      rendy.Cancel();
    }catch(NullPointerException npe){}
    for( int i=0; i<bg.size(); i++ ) 
      try{
	( (DataRenderer)bg.elementAt(i) ).Cancel(); 
      }catch(NullPointerException npe){}
    try{
      gt.dispose();
    }catch(NullPointerException npe){}
    try{
      System.gc();
    }catch(NullPointerException npe){}
    //System.out.println("DBG: Cancel Attempted");
  }
  /*  public void update(Graphics g){
    //g.translate(MaxW+2,FH+2);
    //g.clipRect(MaxW+1,FH+1,wid-2-2*MaxW, heig-2-2*FH);
    System.out.println(getClass() + "  DBG: CanvasView::update(Graphics)      g = " + g.toString());
    Rectangle r = g.getClipRect();
    System.out.println("DBG: x = "+r.x+" y = "+r.y+" width = "+r.width+" height = "+r.height);
    //paint(g);
    super.update(g);
    }
  */
  public void update( Observable obs, Object obj )
    {
      int curcur=hostframe.getCursorType();
      setFrameCursor(Frame.WAIT_CURSOR);
      Cancel();
      repaint(125);
      setFrameCursor(curcur);
    }
}

