/**
 *
 * Mon Apr 13 01:00:00 1998 DataLinearScaleFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/

import java.util.Observable;
import java.util.Observer;

public class DataLinearScaleFilter extends DataFilter
{
  public int [] getDimensions(){
    return in.getDimensions();
  }
	protected DataSource src=null, dest=null;
	protected double[] sub=null, mult=null, add=null, 
	                 cache=null, cachee=null;

	private synchronized void chk()
	{
		if (src==null || dest==null || in==null) return;		
		setChanged(); notifyObservers();
	}
	
	private synchronized void dirty()
	{
		sub = null;
		mult = null;
		add = null;
		cache = null;
		cachee= null;
	}

	
	public synchronized void SetSource( DataSource dv )
	/** Sets the starting box for the linear rescale */
	{
		if (src!=null) src.deleteObserver(this);
		src = dv;
		if (src!=null) src.addObserver(this);
		dirty();
		chk();
	}

	
	public synchronized void SetDestination( DataSource dv )
	/** Sets the corresponding destination box for the linear rescale */
	{
		if (dest!=null) dest.deleteObserver(this);
		dest = dv;
		if (dest!=null) dest.addObserver(this);
		dirty();
		chk();	
	}
	
	public synchronized void SetInput( DataSource _in )
	{
		dirty();
		super.SetInput(_in);
	}
	
	protected synchronized void accum( int i, double sn, double sx, double tn, double tx )
	{
		sub[i] = sn;
		mult[i] = (tx-tn)/(sx-sn);
		add[i] = tn;
		//System.out.println("sub="+sub[i]+" mult="+mult[i]+" add="+add[i]+" tx="+tx+" sx="+sx);
	}
	
	protected synchronized boolean calcScaling()
	{
		if (in==null || src==null || dest==null) return false;
		if (sub==null || mult==null || add==null) { 
			int n = (in.getDimensions())[0];
			double[] s = src.getExtents(); 
			double[] d = dest.getExtents();
			if (s==null || d==null || s.length!=d.length) return false;
			
			sub = new double[ d.length>>1 ];
			mult = new double[ sub.length ];
			add = new double[ sub.length ];
			
			for( int i=0; i<d.length; i+=2 )
				accum( i>>1, s[i], s[i+1], d[i], d[i+1] );
		}
		return true;
	}
  public double[] getData(){ return buildData(in.getData());}
	protected double[] buildData(double[] v)
	{
		if (cache!=null) return cache;
		if (!calcScaling()) return null;
		
		int n = (in.getDimensions())[0]; // tuple width
		if (n>sub.length) n=sub.length; // only scale what we can

		//double[] v = in.getData();			// raw data
		double[] nuv = new double[ v.length ];	// output data
		for( int t=0; t<v.length; t+=n ) // for each tuple 
		  for( int i=0; i<n; i++ )    // and each value in the tuple
		    nuv[t+i] = (v[t+i]-sub[i])*mult[i]+add[i];
		if (cacheEnabled()) cache = nuv;
		return nuv;							
	}
	
	public void update( Observable it, Object data )
	{
	  //System.out.println("DBG: Linear scale filter update called");
		if (it==(Observable)src || it==(Observable)dest) 
			dirty();		// clear out scaling constants and all
		else { cache=null; cachee=null; }
		setChanged(); notifyObservers();
	} 
	
  public double[] getExtents(){ return (dest.getExtents());}
    /*    return buildExtents(in.getExtents());}
  protected double[] buildExtents(double[] e) // min,max per dimension
    {
      //System.out.println("DBG: building extents");
      if (cachee!=null) return cachee;
      if (!calcScaling()) return null;
      int n = (in.getDimensions())[0];
      //double[] e = in.getExtents();
      if (e==null || e.length!=(n*2)) return null;
      
      double[] nue = new double[ e.length ];		
      int j=0;
      for( int i=0; i<n; i++ ) {
	nue[j]=9;
	nue[j] = (e[j]-sub[i])*mult[i]+add[i];
	j++;
	nue[j] = (e[j]-sub[i])*mult[i]+add[i];
	j++;
      }
      if (cacheEnabled()) cachee=nue;
      return nue;
    }*/
}

