/**
 *
 * Fri Apr 24 01:00:00 1998 DataLogScaleFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
import java.util.Observable;

class DataLogScaleFilterDialog extends Frame
{
  private DataLogScaleFilter mom;
  private Button OK, Cancel, Max, Apply, Reset;
  private Panel nsew, buttons;
  private Checkbox x,y,z;
  private boolean ox, oy, oz;
  private int dim;
  public DataLogScaleFilterDialog(DataLogScaleFilter _m, int _d){
    mom = _m;
    dim = _d;
    build();
  }
  public void open(){
    ox= mom.getXBase()!=1.0;
    oy= mom.getYBase()!=1.0;
    oz= mom.getZBase()!=1.0;
    refill();
    //llifer();
    setVisible(true);
  }
  private void refill()
    {
      if (mom==null) return;
      x.setState( mom.getXBase()!=1.0 );
      y.setState( mom.getYBase()!=1.0 );
      z.setState( mom.getZBase()!=1.0 );
    }
  
  private void llifer() // opposite of a refill
    { 
      if (mom==null) return;
      mom.setXState( x.getState() );
      mom.setYState( y.getState() );
      mom.setZState( z.getState() );
    }

  private void build() 
    {
      setLayout( new BorderLayout() );
      setResizable(false);
      nsew = new Panel();
      add( "North", nsew );
      buttons = new Panel();
      add( "South", buttons );
      
      Max = new Button( "Linear" );
      Max.setBackground( Color.orange );
      buttons.add( Max );
      
      OK = new Button( "Okay" );
      OK.setBackground( Color.green );
      buttons.add( OK );
      
      Cancel = new Button( "Cancel" );
      Cancel.setBackground( Color.red );
      buttons.add( Cancel );
      
      Apply = new Button( "Apply" );
      Apply.setBackground( Color.green );
      buttons.add( Apply );
      
      Reset = new Button( "Reset" );
      Reset.setBackground( Color.orange );
      buttons.add( Reset );
      
      nsew.setLayout(new GridLayout(1,dim));
      nsew.add(x = new Checkbox("X-Axis"));
      nsew.add(y = new Checkbox("Y-Axis"));
      z=new Checkbox("Z-Axis");
      if(dim>2)
	nsew.add(z);
      resize(300,150);
      setTitle("Logorithmic Axes");
      nsew.validate();
      buttons.validate();
      validate();
    }
  public boolean handleEvent( Event evt ) 
    {
      if ((evt.id!=Event.ACTION_EVENT) && (evt.key != Event.ENTER))
	return super.handleEvent( evt );
      if (evt.target==(Object)OK) {
	llifer( );
	setVisible(false);
	mom.notifyObservers();
      }
      else if (evt.target==(Object)Apply) {
	llifer( );
	mom.notifyObservers();
	refill();
      }
      else if (evt.target==(Object)Cancel) { 			
	mom.setXState(ox);
	mom.setYState(oy);
	mom.setZState(oz);
	mom.notifyObservers();
	setVisible(false);
      } 
      else if (evt.target==(Object)Max) { 
	mom.setXState(false);
	mom.setYState(false);
	mom.setZState(false);
	refill();
      }
      else if(evt.target==(Object)Reset){
	mom.setXState(ox);
	mom.setYState(oy);
	mom.setZState(oz);
	refill();
      }
      else if(evt.key == evt.ENTER){
	llifer( );
	mom.notifyObservers();
	refill();
      }
      return super.handleEvent( evt );
    }
    }

/**********************************************/

class GroupLogScale extends DataLogScaleFilter
{
  private double [] base = {1.0,1.0,1.0};
  private Vector children = new Vector();
  public void setXBase(double newval){
    for (int z=0; z<children.size(); z++)
      ((DataLogScaleFilter)children.elementAt(z)).setXBase(newval);
    base[0] = newval;
  }    
   public void setYBase(double newval){
    for (int z=0; z<children.size(); z++)
      ((DataLogScaleFilter)children.elementAt(z)).setYBase(newval);
    base[1] = newval;
  }    
   public void setZBase(double newval){
    for (int z=0; z<children.size(); z++)
      ((DataLogScaleFilter)children.elementAt(z)).setZBase(newval);
    base[2] = newval;
  }    
  public synchronized void setXState(boolean n){
      setXBase(n?10:1);
 }
   public synchronized void setYState(boolean n){
      setYBase(n?10:1);
 }
   public synchronized void setZState(boolean n){
      setZBase(n?10:1);
   }
  public void AddFilter(DataLogScaleFilter _new){
    if (_new != null){
      children.addElement(_new);
      _new.ForceLogs(base);
      if (dial==null)
	dial = new DataLogScaleFilterDialog(this,_new.getDimensions()[0]);
    }
  }
  public void RemoveFilter(DataLogScaleFilter _out){
    if (children.contains(_out))
      children.removeElement(_out);
  }
}
/*********************************/

class LogFilter extends DataFilter implements LogCapable, LogBaseCapable{
  protected double[] base={1.0,1.0,1.0}, cache=null;
  protected int dims;
  public boolean invert = false;
  private boolean loghost = false;
  public void SetInput(DataLogScaleFilter n){
    SetInput((DataSource)n);
    //System.out.println("DBG: Log input");
    loghost = true;
  }
  public void SetInput(DataSource n){
    loghost = false;
    if(n==null)
      dims=0;
    else
      dims = n.getDimensions()[0];
    super.SetInput(n);
  }
  public synchronized void setXState(boolean n){
      setXBase(n?10:1);
 }
  public synchronized void setYState(boolean n){
     setYBase(n?10:1);
  }
  public synchronized void setZState(boolean n){
      setZBase(n?10:1);
  }
  public void clearCache(){ cache=null;}
  public synchronized void setXBase(double s){
    /*try{
      System.out.println("DBG: xbase change attempted on object "+in.toString());
    }catch(Exception exp){}*/
     base[0] = s;
     clearCache();
    setChanged(); notifyObservers();
  }
  public double getXBase(){ return base[0];}
  public synchronized void setYBase(double s){
     base[1] = s;
     clearCache();
    setChanged(); notifyObservers();
  }
  public double getYBase(){ return base[1];}
  public synchronized void setZBase(double s){
     base[2] = s;
     clearCache();
    setChanged(); notifyObservers();
  }
  public double getZBase(){ return base[2];}
  public double[] getExtents(){
      double[]out = new double[2*dims], ext = (loghost?(((DataLogScaleFilter)in).sge()):in.getExtents());
      for(int z=0; z<dims;z++)
	for(int y=0;y<=1;y++)
	    out[dims*z+y]=(((base[z]!=1.0)&&(ext[z*dims+y]<=0.0))?(0):
                           (Log(ext[z*dims+y],base[z])));
      return out;
  }
  public int [] getDimensions(){
    return in.getDimensions();
  }
  public double[] getData(){
    if((cache!=null))
      return cache;
      double[]dat = (loghost?(((DataLogScaleFilter)in).sgd()):
		     in.getData()), out = new double[dat.length];
      System.arraycopy(dat,0,out,0,out.length);
      for(int z=0;z<dims;z++)
	if(base[z]!=1.0)
	  for(int y=z;y<dat.length;y+=dims)
	    out[y]=Log(dat[y],base[z]);
      if(cacheEnabled())
	cache=out;
      return out;
  }
  private double Log(double num, double base){
    if (base == 1.0)
      return num;
    if(invert){
      return(Math.pow(base,num));
    }
    if(num<=0.0){
      //System.out.println(Math.pow(10,-320.1));
      return (-320.1);
    }
    return(Math.log(num)/Math.log(base));
  }
  public void update( Observable it, Object data ){
    clearCache();
    setChanged(); notifyObservers();
  } 
}
 
					   

public class DataLogScaleFilter extends DataLinearScaleFilter implements LogCapable, LogBaseCapable
{
  //protected DataLinearScaleFilter dlsf = new DataLinearScaleFilter();
  protected DataLogScaleFilterDialog dial;
  protected LogFilter lf = new LogFilter(), extchld=new LogFilter(), mult = new LogFilter();
  protected DataSource sc=null, star=null, des = null;
  private int dim;
  public boolean invert = false;
  public void open(){
    dial.open();
  }
  public DataLogScaleFilter(){
    mult.invert=true;
  }
  synchronized public void setCaching( boolean s ){
    super.setCaching(s);
     lf.setCaching(s);
     mult.setCaching(s);
     extchld.setCaching(s);    
  }
    
  protected void ForceLogs(double[] _new){
    setXBase(_new[0]);
    setYBase(_new[1]);
    setZBase(_new[2]);
    //extchld.setXBase(_new[0]);
    //extchld.setYBase(_new[1]);
    //extchld.setZBase(_new[2]);
  }
  public void refresh(){
    //ForceLogs();
    SetInput(in);
  }
  public void SetInput(DataSource ds){
    if (ds!=null)
      dim = ds.getDimensions()[0];
    else
      dim = 0;
     sc=ds;
     //super.SetInput(sc);
     //lf.invert = extchld.invert=invert;
      SetSource(star);
      SetDestination(des);
      if(invert){
	super.SetInput(sc);
	mult.SetInput(this);
	//lf.SetInput(des);
	//mult.setInput(
      }else{
	//extchld.SetInput(star);
	//super.SetSource(extchld);
	lf.SetInput(sc);
	super.SetInput(lf);
      }
      if (dial==null)
	dial = new DataLogScaleFilterDialog(this,dim);
  }
  public void SetSource(DataSource ds){
    star = ds;
    if(invert)
      super.SetSource(star);
    else{
      extchld.SetInput(star);
      super.SetSource(extchld);
    }
  }
  public void SetDestination(DataSource ds){
    des = ds;
    if(!invert)
      super.SetDestination(des);
    else{
      lf.SetInput(des);
      super.SetDestination(lf);
    }
  }
  public synchronized void setXBase(double n){
    //System.out.println("DBG: xbase");
      lf.setXBase(n);
      mult.setXBase(n);
      extchld.setXBase(n);
      SetSource(star);
 }
  public synchronized void setYBase(double n){
     lf.setYBase(n);
     mult.setYBase(n);
     extchld.setYBase(n);
      SetSource(star);
  }
  public synchronized void setZBase(double n){
      lf.setZBase(n);
      mult.setZBase(n);
      extchld.setZBase(n);
      SetSource(star);
  }
  public synchronized void setXState(boolean s){
    //System.out.println("DBG: xstate");
    setXBase(s?10:1);
  }
  public double getXBase(){ return   lf.getXBase();}
  public synchronized void setYState(boolean s){
    setYBase(s?10:1);
   }
  public double getYBase(){ return   lf.getYBase();}
  public synchronized void setZState(boolean s){
    setZBase(s?10:1);
  }
  public double getZBase(){ return   lf.getZBase();}
  public double[] getData()
    {
      //refresh();
      //lf.invert =extchld.invert= invert;
      if(invert)
	return(mult.getData());
      else
	return sgd();
    }
  public double[] sgd(){ return super.getData();}
  
  public double[] getExtents(){
      if(invert)
	return(mult.getExtents());
      else
	return sge();
  }
  public double[] sge(){ return super.getExtents();}
  public static void dumparr(double[] arr, String name){
    if(arr==null)
      System.out.println("DBG: "+name+" = null");
    for (int z = 0; z<arr.length; z++)
      System.out.println("DBG: "+name+"["+z+"] = "+arr[z]);
  }
}
