/**
 *
 * Fri Feb 25 12:49:55 2000 DataSource.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/

import java.util.Observable;

public abstract class DataSource extends Observable
{
	public abstract double[] getData();
	public abstract double[] getExtents(); // min,max per dimension
	public abstract int[] getDimensions();

	
	private boolean caching = true;
	
	public boolean cacheEnabled() { return caching; }
	synchronized public void setCaching( boolean s )
	{ caching = s; }
	
	public int tupleCount()
	{
		int[] dim = getDimensions();
		int r = 1;
		for( int i=1; i<dim.length; i++ ) r*=dim[i];
		return r;
	}
}

/**
Data dimensions are recursive tuple style. 
{ 2, 100 } : 100 2-tuples
{ 3, 100, 150 } : 150 100-member vectors of 3-tuples (e.g. 100x150xRGB) 

Extents are for the lowest - order tuple only
e.g. a 3-tuple will return an array of 6 values

*/
