/**
 *
 * Thu Apr 23 01:00:00 1998 DataXYLineRenderer.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Observer;

public class DataXYLineRenderer extends DataRenderer
{
	protected Vector polystore = null;	
	protected Color color = Color.black;
	protected final int MAX_POINTS=64; // max points per segment
	
	public void SetColor( Color c )
	{ if (c!=null) color = c; }
	
	public Color color() 
	{	return color; }

	private void polypaint( Graphics g, Polygon p )
	{
		if ( !g.getClipRect().intersects( p.getBoundingBox() ) ) return;
		for( int i=1; i<p.npoints; i++ )
		{
			 g.drawLine( p.xpoints[i-1], p.ypoints[i-1], 
			 			p.xpoints[i], p.ypoints[i] );
		}
	}

	public void draw( Graphics g )
	{
		Vector ps = polystore;
		if (in==null) return;
		if (ps==null) { calcSegments(); ps = polystore; }
		if (ps==null) return;
		int n = ps.size();
		Object o;
		if (color!=null) g.setColor(color); 
		else g.setColor( Color.black );
		
		for( int i=0; i<n; i++ ) {
			o = ps.elementAt(i);
			if (o instanceof Polygon) {
				polypaint( g, (Polygon)o );
			}
			else if (o instanceof Point) {
				Point p = (Point)o;
				g.drawLine( p.x, p.y, p.x, p.y );
			}
		}
	}
	
	public synchronized void calcSegments()
	{	
		if (in==null) return;
		
		double[] v = in.getData();
		if (v==null) return;

		int state=0;
		boolean notfound;

		int n = (in.getDimensions())[0]; // tuple width
		int x=0,y=0,lx,ly;
		Vector out = new Vector();
		Polygon p = null;
			 
		int i=0;
		for( int t=0; t<v.length; t+=n,i++ ) { // for each tuple 
			lx=x; ly=y;
			if (n==1) {
				notfound = (Double.isNaN(v[t]));
				if (!notfound) {
					x = i;
					y = (int)Math.round(v[t]);
				}
			  } else {
				notfound = (Double.isNaN(v[t]) || Double.isNaN(v[t+1]));
				if (!notfound) {
					x = (int)Math.round(v[t]);
					y = (int)Math.round(v[t+1]);
				}
			}
			
			switch(state) {
				case 0: if (!notfound) state=1;
						break;
						
				case 1: if (notfound) {
							Point pt = new Point( lx, ly );
							out.addElement( pt );
							state = 0;
						  } else { 
							state = 2;
							p = new Polygon();
							p.addPoint( lx, ly );
							p.addPoint( x, y );
						}
						break;
				case 2: if (notfound) {
							out.addElement( p );
							p = null;
							state = 0;
						} else {
							p.addPoint( x, y );
							if (p.npoints>=MAX_POINTS) { 
								out.addElement( p );
								p = null;
								state = 1;
							}
						}
						break;
			}
		}
		if (state==2) {
			out.addElement(p);
			p = null;
		}
		polystore = out;
	}								
	
	private synchronized void dirty()
	{
		polystore = null;
	}
	
	public synchronized void SetInput( DataSource _in )
	{
		dirty();
		super.SetInput( _in );
	}
	
	public void update( Observable it, Object data )
	{
		dirty();
		setChanged(); notifyObservers();
	} 
	
	public double[] getExtents() // min,max per dimension
	{
		return null;
	}

	public int[] getDimensions() 
	{ return null; }
	
}

