/**
 *
 * Thu Mar 12 00:00:00 1998 DotGridRenderer.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Observer;

public class DotGridRenderer extends DataRenderer implements LogCapable
/** 
  *given window extents as input, output a grid of MxN divisions
  */
{
  /**
    * the source for scaling of the numbers
    */
  //protected DataSource source=null;
  /**
    * number of divisions. (spaces)
    */
	protected int xDiv=10, yDiv=10;
  public synchronized void setXBase(double n){
    if (n>=3){
      double []xout = new double[(int)n-1];
      for (int z=1; z<(int)n; z++)
	xout[z-1] = (Math.log(z)/Math.log(n))-1;
      xSubTics = xout;
    } else
      xSubTics = null;
  }
  public synchronized void setYBase(double n){
    if (n>=3){
      double []yout = new double[(int)n-1];
      for (int z=1; z<(int)n; z++)
	yout[z-1] = (Math.log(z)/Math.log(n))-1;
      ySubTics = yout;
    } else
      ySubTics = null;
  }
  public synchronized void setZBase(double n){}
  
  /**
    * These two arrays consist of numbers between -1 and 1, exclusive.
    * These numbers state where between the standard tics are additional
    * tics to be added.  If a value of .5 is entered, a sub tic mark will
    * appear at the halfway point between all of the normal divisional marks.
    * Th reason I suggest <0 values can be used even though they could all be
    * >0 is because if you use a formula, like for adding exponential tics,
    * the tics would be reversed in the case of negative exponents.  Rather
    * than you adjusting your formula to my "standard", just negate it.
    * when not in use, should be set to null. (default)
    */
  public double[] xSubTics = null, ySubTics = null;
  /**
    * colors used for the dotted lines and tic marks
    */
	protected Color c = Color.white ;
  /**
    * Used to set the number of horizontal spaces
    *
    * @param n number of divisions
    */
	public synchronized void SetHorizontalDivisions( int n )
	{ if (n>0) xDiv = n; }

  /**
    * Used to get the number of horizontal spaces
    *
    * @return number of divisions
    */
	public int horizontalDivisions( ) { return xDiv; }
  /**
    * Used to set the number of vertical spaces
    *
    * @param n number of divisions
    */
  	public synchronized void SetVerticalDivisions( int n )
	{ if (n>0) yDiv = n; }
  /**
    * Used to get the number of vertical spaces
    *
    * @return number of divisions
    */
 	public int verticalDivisions( ) { return yDiv; }
  /**
    * sets color of the gridding
    *
    * @param _c the new color
    */
	public void SetColor( Color _c )
        { if (_c!=null) c = _c; }

  /**
    * gets color of the gridding
    *
    * @return the current color
    */
    public Color color()
        {       return c; }
  /**
    * sets the input CanvasDataView from which to get view extents
    *
    * @param _in source for extents
    */
  public synchronized void SetInput(CanvasDataView _in){
    super.SetInput((DataSource) _in);
  }
		
	private final static int HEIGHT=5;
	private final static int WIDTH=5;
  /**
    * standard painting method
    */	public synchronized void draw( Graphics g)
	{
		if (in==null) return;
		double[] ext = in.getExtents();
		if (ext.length<4) return;
		
		double xinc = (ext[1]-ext[0])/(double)xDiv, x=ext[0];
		int ymin = (int)ext[2], ymax = (int)ext[3];
		double yinc = (ext[3]-ext[2])/(double)yDiv, y=ext[2];
		int xmin = (int)ext[0], xmax = (int)ext[1];

		System.out.println("rendering"+ext[0]+" "+ext[1]+" "+ext[2]+" "+ext[3]);
		
		int h = (yinc<0.0)? -HEIGHT : HEIGHT;
		int w = (xinc<0.0)? -WIDTH : WIDTH;
		FontMetrics FM=g.getFontMetrics();
		int FH=FM.getHeight();
		g.setColor(c);
		x=ext[0];
		String output;
		for( int i=0; i<=xDiv+1; x = ext[0]+xinc*(double)i, i++ ) {
		  if (i==xDiv+1) x--;
			g.drawLine( (int)x, ymin, (int)x, ymin+h );
			g.drawLine( (int)x, ymax, (int)x, ymax-h );
			if (xSubTics != null)
			  for (int f = 0; f<xSubTics.length; f++){
			    g.drawLine( (int)(x+xinc*xSubTics[f]), ymin,
					(int)(x+xinc*xSubTics[f]), ymin+h );
			    g.drawLine( (int)(x+xinc*xSubTics[f]), ymax,
					(int)(x+xinc*xSubTics[f]), ymax-h );
			  }
			y = ext[2];
			for( int j=0; j<=yDiv*yDiv; y = ext[2]+(yinc/yDiv)*(double)j, j++ ) {
				g.drawLine( (int)x, (int)y, (int)x, (int)y );		
				// System.out.println( "x,y="+x+","+y );
			}	
		}
		
		y=ext[2];
		for( int i=0; i<=yDiv+1; y = ext[2]+yinc*(double)i-1, i++ ) {
		  if (i==yDiv+1) y+=1;
			g.drawLine( xmin, (int)y, xmin+w, (int)y );
			if (ySubTics != null)
			  for (int f = 0; f<ySubTics.length; f++){
			    g.drawLine( xmin , (int)(y+yinc*ySubTics[f]),
					xmin+w,(int)(y+yinc*ySubTics[f]) );
			    g.drawLine( xmax , (int)(y+yinc*ySubTics[f]),
					xmax-w,(int)(y+yinc*ySubTics[f]) );
			  }
			g.drawLine( xmax, (int)y, xmax-w, (int)y );
			x = ext[0];
			for( int j=0; j<=xDiv*xDiv; x = ext[0]+(xinc/xDiv)*(double)j, j++ ) {
				g.drawLine( (int)x, (int)y, (int)x, (int)y );		
				// System.out.println( "x,y="+x+","+y );
			}	
		}
		g.setColor(Color.white);
		g.drawLine(xmax,ymax,xmin,ymax);
		g.drawLine(xmin,ymin,xmin,ymax);
		g.setColor(Color.gray);
	        g.drawLine(xmax-1,ymax,xmax-1,ymin);
		g.drawLine(xmin,ymin,xmax,ymin);
	}
	
  /**
    * Observable update method
    */
	public void update( Observable it, Object data )
	{
	  //setChanged(); notifyObservers();
	} 
		
}

