/**
 *
 * Fri Apr 24 01:00:00 1998 ExtentsFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
//-----------------------------------------

class ExtentsFilter extends DataFilter
{
	private ExtentsFilterDialog dial = null;
	private DataSource macs = null;
	double [] ext = null;

  public synchronized void dialogDone( double [] _ext) // called by dialog
	{  
	  getExtents();
	  System.arraycopy(_ext,0,ext,0,_ext.length);
	  //if (done) dial = null;
	  setChanged(); 
	  notifyObservers(); 
	}
	public double[] getExtents()
	{
		if (in==null) return null;
		//if (ext==null)
		ext = (double[])in.getExtents();
		return ext;
	}
	
	public void SetMaxSource( DataSource _macs )
	{ macs = _macs; }
	
	public synchronized void Open()
	{
	  getExtents();
	  double [] x = (in==null)? (double[])in.getExtents(): null;
	  //double [] x = (in==null)? (double[])getExtents().clone(): null;
	if ((x==null) && (macs!=null)) x = (double[])macs.getExtents().clone();
	  if (dial == null) dial = new ExtentsFilterDialog( this, x, in, macs, "Change Viewing Extents" );
	  else dial.Open(x);
	}

  public String toString(){
    getExtents();
    String hold = super.toString() +":";
    for (int z=0; z<ext.length; z+=2)
      hold += (" " + (int)ext[z+1] + "/" + (int)ext[z]);
    return hold;
  }

	public void update( Observable obs, Object obj ) 
	{
	  //	  dialogDone(in.getExtents());
	  setChanged(); 
	  notifyObservers(); 
	}

}

class ExtentsFilterDialog extends Frame
/** represents current scale for a given graph
 * when OK is hit, updates itself 
 */
{
	private ExtentsFilter mom;
	
	private Button OK, Cancel, Max, Apply, Reset;
	private Panel nsew, buttons;
	private TextField tfn, tfs, tfe, tfw;
	
	DataSource last, macs;
	double[] extents, oldextents=null;
	int[] dims;
	boolean result = false;
	
	boolean rc() { return result; }
	
	double[] getExtents() 
	{ return extents; }
	
	// later - make it interface to a description source!
	
	public ExtentsFilterDialog( ExtentsFilter _mom, double [] _extents, DataSource _last, DataSource _macs, String title )
	{
		super( title );
		mom = _mom;
		//extents=_extents;
		//if (extents == null) extents = new double[_extents.length];
		//System.arraycopy(_extents,0,extents,0,extents.length);
		last = _last;
		macs = _macs;		
		/*if (last!=null) {
			extents = last.getExtents();
			dims = last.getDimensions();
		}*/
		build();
		Open(_extents);
	}

  public void Open(double[] in)
    {
                                     extents=in;
      		if ((last!=null)&&(extents == null)) {
			extents = last.getExtents();
			dims = last.getDimensions();
		}
		oldextents = new double[extents.length];
		System.arraycopy(extents,0,oldextents,0,extents.length);
		refill();
		resize( 300, 225 );
		setVisible(true);
    }	
	private void refill()
	{
		if (extents==null) return;
		if (extents.length<4) { extents = null; return; }
		tfn.setText( Double.toString( extents[3] ));
		tfs.setText( Double.toString( extents[2] ));
		tfe.setText( Double.toString( extents[1] ));
		tfw.setText( Double.toString( extents[0] ));
	}
	
	private static double fetch( TextComponent it, double old )
	{
		try { 
			return (new Double( it.getText() )).doubleValue();
		} catch ( Exception e ) { 
			return old;
		}
	}
	
	private void llifer() // opposite of a refill
	{ 
		if (extents==null) { 
			extents = new double[4];
			extents[0] = extents[1] = extents[2] = extents[3] = Double.NaN;
		}
		extents[3] = fetch( tfn, extents[3] );
		extents[2] = fetch( tfs, extents[2] );
		extents[1] = fetch( tfe, extents[1] );
		extents[0] = fetch( tfw, extents[0] );
		result = true;
	}

	private void build() 
	{
		setLayout( new BorderLayout() );
		setResizable(false);
		nsew = new Panel();
		add( "North", nsew );
		buttons = new Panel();
		add( "South", buttons );
		
		//plans to add a line-direction  zoom are now possible

		Max = new Button( "Max" );
		Max.setBackground( Color.orange );
		buttons.add( Max );
				
		OK = new Button( "Okay" );
		OK.setBackground( Color.green );
		buttons.add( OK );
		
		Cancel = new Button( "Cancel" );
		Cancel.setBackground( Color.red );
		buttons.add( Cancel );

		Apply = new Button( "Apply" );
		Apply.setBackground( Color.green );
		buttons.add( Apply );
		
		Reset = new Button( "Reset" );
		Reset.setBackground( Color.orange );
		buttons.add( Reset );
		
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		nsew.setLayout( gbl );		
		tfn = new TextField();
		gbc.fill=GridBagConstraints.BOTH;
		gbc.weightx=2;
		gbc.weighty=1;
		gbc.gridwidth=2;
		gbc.gridheight=1;
		gbc.gridx=1;
		gbc.gridy=1;
		gbl.setConstraints(tfn,gbc);
		nsew.add( tfn );
		tfs = new TextField();
		gbc.gridx=1;
		gbc.gridy=3;
		gbl.setConstraints(tfs,gbc);
		nsew.add(  tfs );
		tfw = new TextField();
		gbc.weightx=2;
		gbc.gridwidth=2;
		gbc.gridx=0;
		gbc.gridy=2;
		gbl.setConstraints(tfw,gbc);
		nsew.add( tfw );
		tfe = new TextField();
		gbc.weightx=1;
		gbc.gridwidth=2;
		gbc.gridx=2;
		gbc.gridy=2;
		gbl.setConstraints(tfe,gbc);
		nsew.add( tfe );
		
		Label xmax = new Label("X Max",Label.CENTER),
		  xmin = new Label("X Min",Label.CENTER),
		  ymax = new Label("Y Max",Label.CENTER),
		  ymin = new Label("Y Min",Label.CENTER);
		gbc.gridx=1;
		gbc.gridy=0;
		gbc.weightx=0;
		gbc.gridwidth=2;
		gbl.setConstraints(ymax,gbc);
		nsew.add(ymax);
		gbc.gridy=4;
		gbl.setConstraints(ymin,gbc);
		nsew.add(ymin);
		gbc.gridx=0;
		gbc.gridy=1;
		gbc.gridwidth=1;
		gbc.weightx=0;
		gbl.setConstraints(xmin,gbc);
		nsew.add(xmin);
		gbc.gridx=3;
		gbl.setConstraints(xmax,gbc);
		nsew.add(xmax);
		
	}
		
	public boolean handleEvent( Event evt ) 
	{
	  if ((evt.id!=Event.ACTION_EVENT) && (evt.key != Event.ENTER))
	    return super.handleEvent( evt );
	        if (evt.target==(Object)OK) {
			llifer( );
			setVisible(false);
			mom.dialogDone( extents);
			System.arraycopy(extents,0,oldextents,0,extents.length);
			//dispose();
		}
		else if (evt.target==(Object)Apply) {
			llifer( );
			mom.dialogDone( extents);
			refill();
		}
		else if (evt.target==(Object)Cancel) { 			
			mom.dialogDone( oldextents);
			setVisible(false);
			//dispose();
		} 
		else if (evt.target==(Object)Max) { 
			if (macs!=null) {
				extents = (double[])macs.getExtents().clone();
				refill();
			}
		}
		else if(evt.target==(Object)Reset){
		                 System.arraycopy(oldextents,0,extents,0,extents.length);
		                 refill();
		}
		else if(evt.key == evt.ENTER){
			llifer( );
			mom.dialogDone( extents);
			refill();
		}
		return super.handleEvent( evt );
	}
}


