/**
 *
 * Fri Mar 06 00:00:00 1998 GridRenderer.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Observer;

public class GridRenderer extends DataRenderer
/** given window extents as input, output a grid of MxN divisions
 */
{
	protected int xDiv=1, yDiv=1;
	protected Color c = Color.blue;
	
	public synchronized void SetHorizontalDivisions( int n )
	{ if (n>0) xDiv = n; }

	public int horizontalDivisions( ) { return xDiv; }

	public synchronized void SetVerticalDivisions( int n )
	{ if (n>0) yDiv = n; }

	public int verticalDivisions( ) { return yDiv; }
	
	public void SetColor( Color _c )
        { if (c!=null) c = _c; }

    public Color color()
        {       return c; }
		
	private final static int HEIGHT=5;
	private final static int WIDTH=5;
		
	public void draw( Graphics g )
	{
		if (in==null) return;
		double[] ext = in.getExtents();
		if (ext.length<4) return;
		
		g.setColor(c);
		
		double xinc = (ext[1]-ext[0])/(double)xDiv, x=ext[0];
		int ymin = (int)ext[2], ymax = (int)ext[3];
		double yinc = (ext[3]-ext[2])/(double)yDiv, y=ext[2];
		int xmin = (int)ext[0], xmax = (int)ext[1];

		System.out.println("rendering"+ext[0]+" "+ext[1]+" "+ext[2]+" "+ext[3]);
		
		int h = (yinc<0.0)? -HEIGHT : HEIGHT;
		int w = (xinc<0.0)? -WIDTH : WIDTH;
		
		for( int i=0; i<=xDiv; x = ext[0]+xinc*(double)i, i++ ) {
			g.drawLine( (int)x, ymin, (int)x, ymin+h );
			g.drawLine( (int)x, ymax, (int)x, ymax-h );
		}
		
		
		for( int i=0; i<=yDiv; y = ext[2]+yinc*(double)i, i++ ) {
			g.drawLine( xmin, (int)y, xmin+w, (int)y );
			g.drawLine( xmax, (int)y, xmax-w, (int)y );
			x = ext[0];
			for( int j=0; j<=xDiv; x = ext[0]+xinc*(double)j, j++ ) {
				g.drawLine( (int)x, (int)y, (int)x, (int)y );		
				// System.out.println( "x,y="+x+","+y );
			}	
		}
	}
	
	
	public void update( Observable it, Object data )
	{
		setChanged(); notifyObservers();
	} 
		
}

