/**
 *
 * Fri Apr 10 01:00:00 1998 GroupExtents.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
//-----------------------------------------
/**
  * groups multiple ExtentsFilter's to share one dialog and change together.
  * upon addition to the group, the group extents are copied into the new
  * member, keeping the scale equal to the other members.  Works same if a
  * Static filter is used, but I wouldn't suggest it. (untested)
  *
  * @see GroupNormExtents
  * @see GroupSNormExtents
  */
public class GroupExtents extends ExtentsFilter
{
  private void dumparray(String name, double[] h){
    /*System.out.println("DBG: Dumping array " + name);
    for (int z = 0; z<h.length; z++)
      System.out.println("DBG: array["+z+"] = " + h[z]);
      */
  }
  /**
    * The dialog for this group of extents
    */
  	protected ExtentsFilterDialog dial = null;
  /**
    * The shared extents for this group (post alteration)
    */
	protected double [] ext = null;
  private double[] groupext=null;
  /**
    * the listing of all DataRenderers in this group
    */
        protected Vector in = new Vector();
  /**
    * Opens the corresponding dialog for this object
    */
    
  	public synchronized void Open()
	{
	  if (! in.isEmpty()){
	    double [] x = (((ExtentsFilter)in.firstElement()).getExtents());
	    if (dial == null){
	      BasicDataSource bds = new BasicDataSource();
	      bds.setExtents(allExtents());
	      dial = new ExtentsFilterDialog( this, x, (ExtentsFilter)in.firstElement(),
					      bds, "Change Group Extents");
	    }else dial.Open(x);
	  }
	}
  /**
    * called by the dialog when finished
    *
    * @param _ext  the new extents
    */
  public synchronized void dialogDone( double [] _ext) // called by dialog
	{  
	  //for (int z = 0; z<4 ; z++)
	  //  System.out.println("DBG:   New = " + _ext[z] + "   Old = " +ext[z]);
	  dumparray("into dialog", _ext);
	  /*for (int k=0; k<in.size(); k++)
	    dumparray("Vector in["+k+"]", 
		      ((ExtentsFilter)in.elementAt(k)).getExtents());
		      */
	  System.arraycopy(_ext,0,ext,0,_ext.length);
	  for (int x = 0; x<in.size(); x++)
	    ((ExtentsFilter)in.elementAt(x)).dialogDone((double[])ext.clone());
    	  setChanged(); 
	  notifyObservers(); 
	}
  /**
    * @return the group extents
    */
  public double[] getExtents()
    {
      /*for (int z=0; z<in.size(); z++)
	System.out.println("DBG: in["+z+"]= "+ in.elementAt(z).toString());
      */
      in.trimToSize();
      if (in.isEmpty()) 
	ext = null;
      else 
	//if (ext==null)
	ext = ((ExtentsFilter)in.firstElement()).getExtents();
	    //ext = allExtents();
      /*if(!ext.equals(((ExtentsFilter)in.firstElement()).getExtents()))
	ext=(double[])(((ExtentsFilter)in.firstElement()).getExtents().clone());
      */
      return ext;
    }
  /**
    * @return extents to view all renderers in group
    */
  public synchronized double[] allExtents(){
    if (groupext==null){
      double[] n = new double[4];
      groupext = new double[4];
      System.arraycopy(((ExtentsFilter)in.firstElement()).getExtents(),0,
		       groupext,0, groupext.length);
      for (int z = 0;z<in.size(); z++){
	n = ((ExtentsFilter)in.elementAt(z)).getExtents();
	groupext[0] =	Math.min(groupext[0],Math.min(n[0],n[1]));
	groupext[1] = Math.max(groupext[1],Math.min(n[0],n[1]));
	groupext[2] = Math.min(groupext[2],Math.min(n[2],n[3]));
	groupext[3] = Math.max(groupext[3],Math.min(n[2],n[3]));
      }
    }
    //System.out.println("DBG: GroupExtents Changed");
    return groupext;
  }
  /**
    * updates
    */
	public void update( Observable obs, Object obj ) 
	{
	  //getExtents();
	  setChanged(); 
	  notifyObservers(); 
	}
  /**
    * adds an ExtentsFilter to the group
    *
    * @param _in  the new member
    * @see ExtentsFilter
    */
  public void AddInput(ExtentsFilter _in)
    {
      if (_in != null)
	if (! in.contains(_in)) {
	  _in.addObserver(this);
	  in.addElement(_in);
	  if (in.size()==1) getExtents();
	  groupext=null;
	  dialogDone(allExtents());
	  //((ExtentsFilter)in.lastElement()).dialogDone(getExtents());
	}
      //setChanged(); notifyObservers(); 
    }
  /**
    * removes an ExtentsFilter from the group
    *
    * @param out  the member to be removed
    * @see ExtentsFilter
    */
  public synchronized void RemoveInput( ExtentsFilter out ) 
    {
      if (in.contains(out)){
	in.removeElement(out);
	out.deleteObserver(this);
      }
      getExtents();
    }
  public synchronized int[] getDimensions(){
    return(((ExtentsFilter)in.firstElement()).getDimensions());
  }
}
class BasicDataSource extends DataSource
{
  private double [] data = null, extent = null;
  private  int[] dim = {2,0};
  public double[] getData() { return data; }
  public double[] getExtents() { return extent; }
  public int[] getDimensions() { return dim; }
  public void setData(double [] n) { data = n;  }
  public void setExtents(double[] n) { extent = n; }
  public void setDimensions(int [] n) { dim = n; }
}
