/**
 *
 * Mon Mar 30 00:00:00 1998 GroupNormExtents.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
//-----------------------------------------

/**
  * groups multiple NormExtentsFilter's to share one dialog and change
  * together.  upon addition to the group, the group extents are copied
  * into the new member, keeping the scale equal to the other members.
  * CANNOT be used with the standard ExtentsFilter.
  */
public class GroupNormExtents extends GroupExtents
{
  //private double[] chk = null;
  /**
    * Opens the corresponding dialog for this object
    */
  	public synchronized void Open()
    {
	  if (in.size() > 0){
	    double [] x = (in.size()>0)? (((NormExtentsFilter)in.firstElement()).getNormExtents()): null;
	    //if ((chk == null) && (x!=null)) chk = new double[x.length];
	    if (dial == null)
	      dial = new ExtentsFilterDialog( this, x, (NormExtentsFilter)in.firstElement(),new NormalizedBaseExtents(), "Change Normal Group Extents");
	    else dial.Open(x);
	  }
	}
  /**
    * @return this group's normalized extents
    */
	public double[] getExtents()
	{
	  if (in.size() ==0) return null;
	  if (ext == null) {
	    ext = new double[((NormExtentsFilter)in.firstElement()).getNormExtents().length];
	    System.arraycopy(((NormExtentsFilter)in.firstElement()).getNormExtents(),0,ext,0,ext.length);
	    dialogDone(ext);
	  }
	  return ext;
	}
  /**
    * @return this group's normalized extents
    */
	public double[] getNormExtents()
	{
	  return getExtents();
	}
  /**
    * Adds a NormExtentsFilter to the group
    *
    * @param _in  the member to be added
    * @see NormExtentsFilter
    */
  public void AddInput(NormExtentsFilter _in)
    {
      if (_in != null)
	if (! in.contains(_in)) {
	  in.addElement(_in);
	  _in.addObserver(this);
	  _in.dialogDone(getExtents());
	}
      setChanged(); notifyObservers(ext); 
    }
  /**
    * Removes a NormExtentsFilter from the group
    *
    * @param out  the member to be removed
    * @see NormExtentsFilter
    */
  public synchronized void RemoveInput( NormExtentsFilter out ) 
    {
      if (in.contains(out)){
	in.removeElement(out);
	out.deleteObserver(this);
      }
      getExtents();
    }
}
