/**
 *
 * Thu Apr 02 00:00:00 1998 GroupSNormExtents.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;
//-----------------------------------------

/**
  * groups multiple SNormExtentsFilter's to share one dialog and change
  * together.  upon addition to the group, the group extents are copied
  * into the new member, keeping the scale equal to the other members.
  * CANNOT be used with either NormExtentsFilters nor ExtentsFilters.
  * The needs of a SNormExtentsFilter is a little different from Norm.
  */
public class GroupSNormExtents extends GroupNormExtents implements LogCapable
{
  private boolean[] bases = {false,false,false};
  public void setXState(boolean n){
    for (int x = 0; x<in.size(); x++)
      ((SNormExtentsFilter)in.elementAt(x)).setXState(n);
    bases[0]=n;
  }
  public void setYState(boolean n){
    for (int x = 0; x<in.size(); x++)
      ((SNormExtentsFilter)in.elementAt(x)).setYState(n);
    bases[1]=n;
  }
  public void setZState(boolean n){
    for (int x = 0; x<in.size(); x++)
      ((SNormExtentsFilter)in.elementAt(x)).setZState(n);
    bases[2]=n;
  }

  /**
    * Used by the dialog
    */
  public synchronized void dialogDone( double [] _ext) // called by dialog
	{  
	  //if (!(_ext.equals(new NormalizedBaseExtents().getExtents()))){
	    for (int x = 0; x<in.size(); x++)
	      ((SNormExtentsFilter)in.elementAt(x)).dialogDone((double[])_ext.clone());
	    System.arraycopy(new NormalizedBaseExtents().getExtents(),0,_ext,0,_ext.length);
	    setChanged(); 
	    notifyObservers(); 
	}
  /**
    * Adds a SNormExtentsFilter to this group
    *
    * @param _in the member to be added
    * @see SNormExtentsFilter
    */
  public void AddInput(SNormExtentsFilter _in)
    {
      if (_in != null)
	if (! in.contains(_in)) {
	  in.addElement(_in);
	  _in.addObserver(this);
	  _in.dialogDone(getExtents());
	  _in.setXState(bases[0]);
	  _in.setYState(bases[1]);
	  _in.setZState(bases[2]);
	}
      setChanged(); notifyObservers(); 
    }
  /**
    * Removes a SNormExtentsFilter from this group
    *
    * @param out the member to be added
    * @see SNormExtentsFilter
    */
  public synchronized void RemoveInput( SNormExtentsFilter out ) 
    {
      if (in.contains(out)){
	in.removeElement(out);
	out.deleteObserver(this);
      }
      getExtents();
    }
}
