/**
 *
 * Fri Mar 06 00:00:00 1998 KeySubRenderer.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
/**
  * Used by the GroupRenderer class to draw a key.  Rewrites are welcome.
  * This is not meant to be a full public class, but a member for GroupRenderer.
  */
public class KeySubRenderer
{
  /**
    * the String Labels for each object in the host GroupRenderer.  indexes are
    * synchronized with the host object.
    */
  protected Vector data = new Vector();
  /**
    * the font used by the key.
    */
  protected Font font = new Font("Times", Font.PLAIN, 18);
  /**
    * the coords for the upper left hand corner of the key's "box"
    */
  protected int x=11,y=10;
  /**
    * used by GroupRenderer to paint a label
    *
    * @param g     the host graphics object
    * @param graph the renderer's index from which to get the label
    * @param loc   the location on the key, (1=top,2=next)
    */
  public synchronized void draw (Graphics g, int graph, int loc)
    {
      g.setFont(font);
      FontMetrics FM = g.getFontMetrics();
      int FH=FM.getHeight();
      if (((String)data.elementAt(graph)) != ""){
	g.fill3DRect(x, loc*FH + y + (FH/4) , FH/2, FH/2,true);
	g.drawString(((String)data.elementAt(graph)),x+3*FH/4,y+loc*FH + 3*FH/4);
      }
      //g.setColor(Color.black);
      //g.drawRect(-999,10,9999,FH*(loc+1));
    }
  /**
    * @return font used by the key
    */
  public synchronized Font GetFont() { return font; }
  /**
    * used by GroupRenderer to set a font
    *
    * @param _font the new font
    */
  public synchronized void SetFont(Font _font) {font = _font;}
  /**
    * set font name to make new key font
    *
    * @param name new font name
    */
  public synchronized void SetFontName(String name) {
    font = new Font(name, font.getStyle(), font.getSize());
  }
  /**
    * set font style to make new key font
    *
    * @param styl new font style
    */
  public synchronized void SetFontStyle(int styl) {
    font = new Font(font.getName(), styl, font.getSize());
  }
  /**
    * set font size to make new key font
    *
    * @param siz new font size
    */
  public synchronized void SetFontSize(int siz) {
    font = new Font(font.getName(), font.getStyle(), siz);
  }
  /**
    * @return  label at idx x
    * @param x index
    */
  public synchronized String GetLabelAt(int x){ return ((String)data.elementAt(x)); }
  /**
    * sets a label at an index (used by GR)
    *
    * @param l the new label
    * @param x the index
    */
  public synchronized void SetLabelAt(String l, int x) {data.setElementAt(l,x);}
  /**
    * adds a label (used by GR)
    *
    * @param x new label
    */
  public synchronized void AddLabel(String x) { data.addElement(x);}
  /**
    * removes a label (used by GR)
    *
    *@param x index of label
    */
  public synchronized void RemoveLabelAt(int x) { data.removeElementAt(x); }
  /**
    * sets the upper left hand corner location of the key (used by GR)
    *
    * @param _x x coord from left side of graph > 0
    * @param _y y coord from top of graph > 0
    */
  public synchronized void SetLocation(int _x, int _y) { x=_x; y=_y;}
}

