/**
 *
 * Fri Apr 17 01:00:00 1998 LabelAdapter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observer;
import java.util.Observable;


class LabelAdapter implements Observer
/** This class updates text controls whenever a view extent changes 
    It also can be attached to a cursor position label, though you probably 
    don't want to do both at once since your extent labels get updated whenever
    your cursor moves (yuk!) */
{
	private DataSource view=null;
	private DataDescSource desc=null;
	private Component tr=null, bl=null, cursor=null;
	private double[] old={Double.MAX_VALUE,Double.MAX_VALUE,Double.MAX_VALUE}, spc={1,1,1};
	private boolean staticround=false;
	public boolean round=false;
	public void setRound(int digs){
	  double ten = Math.pow(10,digs);
	  staticround=true;
	  for (int z=0;z<3; z++)
	    spc[z]=ten;
	}
	public String left = " ", right = " ";
	
	public synchronized void SetInput( DataSource _view )
	{ if (view!=null) view.deleteObserver(this); 
	  view = _view; 
	  if (view!=null) {
	    view.addObserver(this);
	    update( view, null ); 
	  }
	}
	
	public synchronized void SetFormat( DataDescSource _desc )
	{ if (desc!=null) desc.deleteObserver(this); 
	  desc = _desc; 
	  if (desc!=null) {
	    desc.addObserver(this);
	    update( desc, null ); 
	  }
	}
	
	public synchronized void AttachTopRight( Component _tr )
	{ tr = _tr; if (tr!=null) update( null, null ); }
	
	public synchronized void AttachBottomLeft( Component _bl )
	{ bl = _bl; if (bl!=null) update( null, null ); }
	
	public synchronized void AttachCursor( Component _cursor )
	  { cursor = _cursor; if (cursor!=null) update( null, null ); }
	
	private String format( int offset, double[] data )
				/* negative offset implies data, not extents */
	  {
	    int n = data.length;
	    if (n==0) return "";
	    
	    int dex = 2;
	    if (offset<0) { offset=0; dex=1; }
	    else n/=2;
	    
	    String out = "(";
	    for( int i=0; i<n; i++ ) {
	      // add code for different formatsrlog
	      if (!round)
		out =out+(data[i*dex+offset])+ (n-1==i? ")":",");
	      else{
		if((n<=3)&&(!staticround)){
		  double diff = (Math.abs(old[i]-data[i*dex+offset]));
		  //System.out.println("DBG: "+diff);
		  if(diff!=0)
		    if(diff<1){
		      //System.out.println("DBG: loc check");
		      spc[i] = Math.pow(10,Math.round(Math.log(1/diff)/Math.log(10)));
		    }else{
		      spc[i]=1;
		    }
		  old[i]=data[i*dex+offset];
		}
		out = out+((double)((int)(data[i*dex+offset]*spc[i]))/spc[i])+ (n-1==i? ")":",");
	      }
	    }
	    //}
	    out = left +out +right;
	    return out;
	  }
	
	public void update( Observable ob, Object pv )
	  {
	    if (view==null) return;
	    double[] ext = null;
	    if (tr!=null || bl!=null) ext = view.getExtents();
	    
	    if ((tr!=null) && (ext!=null)) {
	      if (tr instanceof TextComponent) 
		((TextComponent)tr).setText( format( 1, ext ) );
	      else if (tr instanceof Label) 
		((Label)tr).setText( format( 1, ext ) );
	    }
	    if ((bl!=null) && (ext!=null)) {
	      if (bl instanceof TextComponent) 
		((TextComponent)bl).setText( format( 0, ext ) );
	      else if (bl instanceof Label) 
		((Label)bl).setText( format( 0, ext ) );
	    }		
	    if (cursor!=null) {
	      double[] data = view.getData();
	      if (data==null) return;
	      if (cursor instanceof TextComponent) 
		((TextComponent)cursor).setText( format( -1, data ) );
	      else if (cursor instanceof Label) 
		((Label)cursor).setText( format( -1, data ) );
	    }
	    
	  }
}
