/**
 *
 * Mon Apr 27 01:00:00 1998 MergedDataSource.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.util.Observer;
import java.util.Observable;
/**
  * Creates a full DataSource object from 2 or 3 double arrays
  */
public class MergedDataSource extends DataSource
{
  public static double[] sum(double[] a,double[]b){
    int count = Math.min(a.length, b.length);
    double hold[] = new double[count];
    for (int z=0;z<count;z++)
      hold[z] = a[z]+b[z];
    return hold;
  }
  public static double[] difference(double[] a,double[]b){
    int count = Math.min(a.length, b.length);
    double hold[] = new double[count];
    for (int z=0;z<count;z++)
      hold[z] = a[z]-b[z];
    return hold;
  }
  public static double[] product(double[] a,double[]b){
    int count = Math.min(a.length, b.length);
    double hold[] = new double[count];
    for (int z=0;z<count;z++)
      hold[z] = a[z]*b[z];
    return hold;
  }
  public static double[] divis(double[] a,double[]b){
    int count = Math.min(a.length, b.length);
    double hold[] = new double[count];
    for (int z=0;z<count;z++)
      hold[z] = a[z]/b[z];
    return hold;
  }
  public static double[] power(double[] a,double[]b){
    int count = Math.min(a.length, b.length);
    double hold[] = new double[count];
    for (int z=0;z<count;z++)
      hold[z] = Math.pow(a[z],b[z]);
    return hold;
  }
  private double [] data = null;
  private double [] extent = null;
  private int[] dim = null;
  /** 
    * Constructor
    * takes two double arrays and becomes a full 2D DataSource
    *
    * @param x the array to be used as the x coords
    * @param y the array to be used as the y coords
    */
  public MergedDataSource (double [] x, double[] y){
    if ((x.length !=0) && (y.length!=0)){
      dim = new int[2];
      extent = new double[4];
      data = new double[Math.min(x.length,y.length)*2];
      extent[0] = extent[1] = x[0];
      extent[2] = extent[3] = y[0];
      for (int c=0; c<Math.min(x.length,y.length); c++){
	data[2*c] = x[c];
	data[2*c+1] = y[c];
	extent[0] = Math.min(extent[0],x[c]);
	extent[1] = Math.max(extent[1],x[c]);
	extent[2] = Math.min(extent[2],y[c]);
	extent[3] = Math.max(extent[3],y[c]);
      }
      dim[0]=2;dim[1]=(data.length)/dim[0];
    } else {
      System.out.println("No Input. Null value set.");
    }
  }
  /** 
    * Constructor
    * takes three double arrays and becomes a full 3D DataSource
    *
    * @param x the array to be used as the x coords
    * @param y the array to be used as the y coords
    * @param z the array to be used as the z coords
    */
  public MergedDataSource (double [] x, double[] y, double[] z){
    if ((x.length !=0) && (y.length!=0) && (z.length!=0)){
      dim = new int[2];
      extent = new double[6];
      data = new double[Math.min(z.length,Math.min(x.length,y.length))*3];
      extent[0] = extent[1] = x[0];
      extent[2] = extent[3] = y[0];
      extent[4] = extent[5] = z[0];
      for (int c=0; c<Math.min(z.length,Math.min(x.length,y.length)); c++){
	data[3*c] = x[c];
	data[3*c+1] = y[c];
	data[3*c+2] = z[c];
	extent[0] = Math.min(extent[0],x[c]);
	extent[1] = Math.max(extent[1],x[c]);
	extent[2] = Math.min(extent[2],y[c]);
	extent[3] = Math.max(extent[3],y[c]);
	extent[4] = Math.min(extent[4],z[c]);
	extent[5] = Math.max(extent[5],z[c]);
      }
      dim[0]=3;dim[1]=(data.length)/dim[0];
    } else {
      System.out.println("No Input. Null value set.");
    }
  }
  /**
    * inherited method from DataSource
    */
  public synchronized double[] getData() { return data; }
  /**
    * inherited method from DataSource
    */
  public synchronized double[] getExtents() { return extent; }
  /**
    * inherited method from DataSource
    */
  public synchronized int[] getDimensions() { return dim; }
}
