/**
 *
 * Fri Feb 25 12:49:56 2000 MuxFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.util.Hashtable;
import java.util.Observable;
import java.util.Observer;

public class MuxFilter extends DataFilter
/** Given: a set of triggers and corresponding DataSources
    When a given trigger changes, Mux takes on the value of the corresponding
    source. 
    useful for eliminating infinite feed back loops 
    useful for sending several sources (e.g. scales) into one stack/buffer
  */
{
	Hashtable select2source = new Hashtable();
		
	public synchronized void RemoveInput( Observable _in )
	/** clears given the trigger, since the sources may have multiple triggers
	     but triggers may not have multiple sources */
	{
		select2source.remove( (Object)_in );
	}
	
	private synchronized void select( DataSource it )
	{
		in = it;
		setChanged(); notifyObservers();
	}
	
	public synchronized void Select( DataSource _in )
	{
		if (select2source.contains( _in )) select( _in );
	}
	
	public synchronized void Select( Observable _in )
	{
		Object o = select2source.get( _in );
		if (o==null) return;
		select( (DataSource)o );			
	}
	
	public synchronized void AddInput( DataSource _in, Observable _trig )
	{
		select2source.put( _trig, _in );
	}
	
	public void SetInput( DataSource _in )
	{  /* donothing */
	}

	public double[] getData() 
	{ if (in!=null) return in.getData();  return null; }
	public double[] getExtents()
	{ if (in!=null) return in.getExtents(); return null; }
	public int[] getDimensions()
	{ if (in!=null) return in.getDimensions(); return null; }
		
	public void update( Observable o, Object arg )
	{
		if (o==(Observable)in) { setChanged(); notifyObservers(); }
		else Select( o );
	}
}

