/**
 *
 * Mon Mar 30 00:00:00 1998 NormExtentsFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
//-----------------------------------------
/**
  * The Normalized Base Extents.  currently are [-1,1]
  */
class NormalizedBaseExtents extends DataSource
{
  /**
    * the Normalized Extents
    */
	public double [] extent = { -1,1, -1,1 };
  /**
    * dims of 2, 0 for data
    */
	public int[] dim = { 2,0  };
  /**
    * @return null
    */
 	public double[] getData() { return null; }
  /**
    * @return normalized extents
    */
	public double[] getExtents() { return extent; }
  /**
    * @return base dimensions
    */
	public int[] getDimensions() { return dim; }
}
/**
  * This class makes understanding relative extents easier.  Upon 
  * instantiation, this class copies the parent's extents.  This is set
  * as the base extents.  The dialog then uses the min -1 and max 1 as a
  * set of Normalized base extents (from the NormalizedBaseExtents class)
  * the numbers correspond directly to the actual extents of the object.
  * Suppose you were to change the X extents from -1,1 to -.5,5?  What would
  * happen?  A zoom in to 50%.  If you changed it back to -1,1, it would be
  * back to the base.  a -1,1 to 1,-1 would flip.  a -1,1 to 0,2 would pan.
  * and so on.  Very versitile class for standardizing extents.
  */
public class NormExtentsFilter extends ExtentsFilter
{
  /**
    * dialog for this object
    */
  	protected ExtentsFilterDialog dial = null;
  //              private DataSource outp = null;
  private double [] Next = null;
  private double [] inext = null;
  protected DataSource macs = new NormalizedBaseExtents(), 
    self = new NormalizedBaseExtents();
  /**
    * This is what makes this class tick.  The Normalized Extents are scaled
    * here, using a copy of the original isnatantiated extents.
    *
    * @param newext the new extents in normalized notation
    */
  protected synchronized void Rescale(double[] newext) 
    {
      if (newext != null){
	double [] repext =  (double[])new NormalizedBaseExtents().getExtents(), outext = in.getExtents();
	// t = template, n=new, o=old, c=coord (N/A), l=length
	// Normalized extents have a standard old max of 1 and min of -1. (pretty normal, eh?)
	  double ol, tnl, tol, tc;
	  for(int d=0; d<inext.length; d+=2){
	    ol = (inext[d+1]-inext[d]);
	    tol = (repext[d+1]-repext[d]);
	    tnl = (newext[d+1]-newext[d]);
	    if ((tnl != 0) && (tol != 0)) {
	      if (tol != tnl){
		outext[d] = ((newext[d] - repext[d])/(tnl-tol))*(ol*(tnl/tol)-ol) + inext[d];
		outext[d+1] = ((newext[d+1] - repext[d+1])/(tnl-tol))*(ol*(tnl/tol)-ol) + inext[d+1];
	      } else {
		outext[d] = ((newext[d] - repext[d])*(ol/tol)) + inext[d];
		outext[d+1] = ((newext[d+1] - repext[d+1])*(ol/tol)) + inext[d+1];
	      }
	    // now wasn't that beautiful?
	    /* Explanation of how Rescale should work:
	       The first term tells what percent of the change goes to this term. the second is the actual length change.  The change*percent give the amount change to this term to scale it correctly. This rewrites the datasource's extents using the scale and exits.*/
	    }
	  }
      }
    }
  /**
    * The reverse of Rescale().  this takes the current extents, and models the
    * normalized extents accoringly.
    */
   protected synchronized void elacseR() 
    {
      double [] newext =  in.getExtents();
      double [] repext =  (double[])new NormalizedBaseExtents().getExtents();
      double ol, tnl, tol;
      for(int d=0; d<inext.length; d+=2){
	ol = (repext[d+1]-repext[d]);
	tol = (inext[d+1]-inext[d]);
	tnl = (newext[d+1]-newext[d]);
	if ((tnl != 0) && (tol != 0)) {
	  if (tol != tnl) {
	    Next[d] = ((newext[d] - inext[d])/(tnl-tol))*(ol*(tnl/tol)-ol) + repext[d];
	    Next[d+1] = ((newext[d+1] - inext[d+1])/(tnl-tol))*(ol*(tnl/tol)-ol) + repext[d+1];
	  } else {
	    Next[d] = ((newext[d] - inext[d])*(ol/tol)) + repext[d];
	    Next[d+1] = ((newext[d+1] - inext[d+1])*(ol/tol)) + repext[d+1];
	  }
	}
      }
    }
  /**
    * Opens the dialog
    */
  	public synchronized void Open()
	{
	  getExtents();
	  elacseR();
	  double [] x = (self==null)? ((double[])self.getExtents()): null;
	  if ((x==null) && (macs!=null)) x = ((double[])macs.getExtents().clone());
	  if (dial == null) dial = new ExtentsFilterDialog( this, x, self, macs, "Normalized Extents" );
	  else dial.Open(x);
	}
  /**
    * Sets the input DataSource for this object
    *
    * @param n the input object
    */
  public void SetInput(DataSource n)
    {
      if (n == null) {
	inext = null;
	Next = null;
      } else {
	inext = new double[n.getExtents().length];
	System.arraycopy(n.getExtents(),0,inext,0,inext.length);
	self = new NormalizedBaseExtents();
	Next = self.getExtents();
      }	
      if (in!=null) in.deleteObserver(this);
      in = n; 
      if (in!=null) {
	setCaching(in.cacheEnabled()); 
	in.addObserver(this);
      }
      setChanged(); notifyObservers(); 
    }
  /**
    * called by the dialog to change the extents
    *
    * @param _ext the new normalized extents
    */
  public synchronized void dialogDone( double [] _ext) // called by dialog
	{  
	  Rescale(_ext);
	  setChanged(); 
	  notifyObservers(); 
	}
  /**
    * @return the objects current normalized extents
    */
	public double[] getNormExtents()
	{
		if (self==null) return null;
		if (Next == null){
		  self = new NormalizedBaseExtents();
		  Next = self.getExtents();
		}
		return Next;
	}
  public double[] getBaseExtents() {return (double[])inext.clone();}
}
