/**
 *
 * Fri Mar 06 00:00:00 1998 OptimizeExtentsFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.util.Observable;
/**
  * This class is meant only as an initial tweak.  Once this is used, the filter is dormant.  Any
  * changes are ignored, all further extents changes are at the user's discretion.  I repeat:
  * THIS FILTER DOES ITS JOB ONCE, AND NO MORE.
  */
public class OptimizeExtentsFilter extends DataFilter
{
  //private boolean done=false;
  private double[] output = null;
  /**
    * array of booleans stating whether or not an axis is logorithmic.  default = false
    */
  public boolean[] power = {false, false, false};
  /**
    * This is an array of the bases used for rating the labels.  In the case of a logorithmic graph,
    * the filter will make sure each label is a number that is a whole number power of the base.
    * If a linear graph, each label is a multiple of base.
    */
  public double[] base = {10,10,10};
  /**
    * Array of how many SPACES you intend to have, so this knows what NICE is defined as.
    * If set to 10 (default), it thinks you intend to have: 9 lines on the graph (11 if you include
    * the edges of the graph) and 10 spaces.  this is the space count.
    */
  public int[] Divs = {10,10,10};
  /**
    * This boolean can make or break a good graph.  If false (default), this filter will try not to
    * trim the extents to exclude some that would have otherwise been there.  If true, it may trim
    * off quite a bit of the graph.  If it is too extreme in either direction, adjust the Divs array or
    * the base array to reduce the resolution it knows you like or the steps for each div respectively.
    * I beleive that the way i have it written, if min > max, the above definition is reversed
    */
  public boolean exclusive = false;
  public double[] getExtents(){
    //if ((power[0] == pbkp[0]) &&(power[1] == pbkp[1]) &&(power[2] == pbkp[2]))
    //return output;
    if (output != null)
      return output;
    //System.arraycopy(power,0,pbkp,0,pbkp.length);
    double[] input = in.getExtents();//, output;
    if (input == null)
      return null;
    output = new double[input.length];
    System.arraycopy(input,0,output,0,output.length);
    for (int z=0; z<input.length; z+=2){
      if (!power[z/2]){
	int lower = (int)(input[z])%((int)base[z/2]);
	if (lower != 0){
	  output[z] = (double)((int)(input[z]) - lower);
	  if (!exclusive)
	    output[z] -= base[z/2];
	}
	int width = ((int)input[z+1] - (int)output[z])%((int)base[z/2]*(Divs[z/2]));
	//System.out.println("DBG: Width = " + width);
	if (width != 0){
	  output[z+1] = (double)(((int)input[z+1]) - width);
	  if (!exclusive)
	    output[z+1] += ((int)base[z/2])*(Divs[z/2]);
	}
      }else{
	double lower =baselog(input[z],base[z/2]);
	if (exclusive)
	  output[z] = Math.pow(base[z/2], (int)(lower)+1);
	else
	  output[z] = Math.pow(base[z/2], (int)(lower));
	int width = (int)(baselog(input[z+1],base[z/2])-baselog(output[z],base[z/2])),
	  widexp = width%Divs[z/2];
	if (widexp !=0){
	  if (exclusive)
	    output[z+1] = Math.pow(base[z/2],(int)(width)-widexp+3+(int)lower);
	  else
	    output[z+1] = Math.pow(base[z/2],(int)(width)-widexp + Divs[z/2]+(int)lower-1);
	}
      }
    }
    //setChanged(); notifyObservers();
    return output;
  }
  private double baselog(double l, double b){
    if (l>0)
      return Math.log(l)/Math.log(b);
    else
      return 0;
  }
  public void update( Observable obs, Object obj ){
    //System.out.println("DBG: UPDATE CALLED");
    setChanged();
    notifyObservers();
  }
}
