/**
 *
 * Mon Apr 27 01:00:00 1998 RegRuledCanvasView.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import java.util.Date;

class RegRuledCanvasView extends CanvasView implements LogCapable{
  private RegRuledCanvasView loop = this;
  protected int wid=100,heig=100;
  private int tick = 4;
  public synchronized void setXState(boolean n){ 
    xlog = n;
    setExtents(in);
  }
  public synchronized void setYState(boolean n){ 
    ylog = n;
    setExtents(in);
  }
  public synchronized void setZState(boolean n){}
  public synchronized boolean getXState(){ return xlog;}
  public synchronized boolean getYState(){ return ylog;}
  //public synchronized double getZBase(){ return 1;}
  private boolean xlog = false, ylog = false;
  public void setColor(Color b){fgrnd=b;}
  //public void setRulerBackground(Color b){bgrnd=b;}
  private Color fgrnd=Color.black, bgrnd=Color.lightGray;
  protected Font font = new Font("Roman", Font.PLAIN, 10);
  protected FontMetrics FM;
  protected int FH =-1, MaxW=-1, inx=0, iny=0;
  protected DataSource in=null;
  protected double[] ext=null;
  public boolean redraw(){
    if (FM == null){
      setVisible(true);
      if (getGraphics() != null){
	FM = getGraphics().getFontMetrics(font);
	FH = FM.getHeight();
      }
    }
    boolean bufff = false;
    ext = in.getExtents();
    if (FM != null){
      /*int n=(Math.max(FM.stringWidth(new Integer((int)ext[2]).toString()),
		     FM.stringWidth(new Integer((int)ext[3]).toString())));*/
      //System.out.println("DBG: "+(new Double(ext[2]).toString()));
      int n=(Math.max(FM.stringWidth(new Integer((int)(ext[2]*100)).toString()),
		     FM.stringWidth(new Integer((int)(ext[3]*100)).toString())));
      if ((ylog)&&((Log(ext[3])<0)||(Log(ext[2])<0)))
	n += FM.stringWidth("22");
      if(n!= loop.MaxW)
	bufff = true;
      MaxW = n;
      loop.MaxW = n;
      super.reshape(inx+MaxW+2,iny+FH+2,wid-4-2*MaxW,heig-4-2*FH);
      //System.out.println("font width = " + MaxW + " height = " +FH);
      //}
      return bufff;
    }
    return false;
  }
  protected boolean arraycomp (double[] a, double [] b, int idx){
    if (a.length != b.length)
      return false;
    if (a.length == idx)
      return true;
    if (a[idx] == b[idx])
      return (arraycomp(a,b,idx+1));
    return false;
  }
  public void setBounds( int x, int y, int _w, int _h) {reshape(x,y,_w,_h);}
  public void setBounds( Rectangle r){ reshape(r.x,r.y,r.width,r.height);}
  public void reshape( int x, int y, int _w, int _h ){
    inx = x;
    iny=y;
    wid = _w;
    heig = _h;
    redraw();
    canvasreshape(x,y,_w,_h);
  }
  public void resize(int w, int h){
    reshape(0,0,w,h);
  }
  private final double diff = .25;
  private Color light (Color clr){
    return(new Color((int)(clr.getRed()*(1+diff)),(int)(clr.getGreen()*(1+diff)),
		     (int)(clr.getBlue()*(1+diff))));
  }
  private Color dark (Color clr){
    return(new Color((int)(clr.getRed()*(1-diff)),(int)(clr.getGreen()*(1-diff)),
		     (int)(clr.getBlue()*(1-diff))));
  }
  public void setBackgroundColor(Color c){bgrnd=c;}
  public void setRulerBackColor(Color c){super.setBackgroundColor(c);}
  public void background(Graphics g){
    super.background(g);
    g.setColor(bgrnd);
    g.fillRect(MaxW+2,FH+2,wid-4-2*MaxW,heig-4-2*FH);
    /*
      g.setColor(dark(getBackground()));
      g.drawLine(MaxW+2,FH+2,wid-2-MaxW,FH+2);
      g.drawLine(MaxW+2,FH+2,MaxW+2,heig-2-FH);
      g.setColor(light(getBackground()));
      g.drawLine(wid-2-MaxW,heig-2-FH,2+MaxW,heig-2-FH);
      g.drawLine(wid-2-MaxW,heig-2-FH,wid-2-MaxW,2+FH);
    */
  }
  public synchronized void draw(Graphics g){
    g.clipRect(1,1,wid,heig);
    g.setFont(font);
    g.setColor(fgrnd);
    double[] ar = new double[ext.length];
    System.arraycopy(ext,0,ar,0,ar.length);
    if(xlog){
      ar[0]=Log(ar[0]);
      ar[1]=Log(ar[1]);
    }
    if(ylog){
      ar[2]=Log(ar[2]);
      ar[3]=Log(ar[3]);
    }
    if (FM == null){
      setVisible(true);
      if (g != null){
	FM = g.getFontMetrics();
	FH = FM.getHeight();
      }
    }
    double[]fr = loop.in.getExtents();
    if (FM != null){
      int n=(Math.max(FM.stringWidth(new Integer((int)(fr[2]*100)).toString()),
		      FM.stringWidth(new Integer((int)(fr[3]*100)).toString())));
      if ((ylog)&&((Log(fr[3])<0)||(Log(fr[2])<0)))
	n += FM.stringWidth("22");
      if ((n!=MaxW) || (fr[2] != ext[2]) || (fr[3] != ext[3]) || (fr[0] != ext[0]) || (fr[1] != ext[1])){
	loop.reshape(inx,iny,wid,heig);
	return;
      }
    }
    g.translate(MaxW+2, FH+2);
    Graphics gout = g.create(0,0,wid-4-2*MaxW, heig-4-2*FH);
    Rectangle r = gout.getClipRect();
    /********/
    //int c =(int)(Log(ext[1]-ext[0]) -1);
    int hmw=(Math.max(FM.stringWidth(new Integer((int)(ext[0]*100)).toString()),
		      FM.stringWidth(new Integer((int)(ext[1]*100)).toString())));
    double addon = /*(xlog?(Math.pow(10,(int)Log((int)(ar[1]-ar[0])))):*/
		    ((ar[1]-ar[0]<0)?-1:1)*(Math.pow(10,(int)(Log(Math.abs(ar[1]-ar[0])) -1)));
    //if(!xlog)
    //addon = addon*5;
    if(xlog)
      hmw*=1.5;
    if((((wid-4-2*MaxW)/hmw)<((ar[1]-ar[0])/addon)))
      if(!xlog){
	boolean fiver = true;
	while(((wid-4-2*MaxW)/hmw)<((ar[1]-ar[0])/addon)){
	  addon = addon*(fiver?5:2);
	  if(Math.abs(ar[1]-ar[0])<Math.abs(addon*(fiver?2:5)))
	    break;
	  fiver = !fiver;
	}
      }else
	addon =addon*10;
    double coord=/*(xlog?Math.round(ar[0]):*/(((int)((ar[0])/addon))*addon),
      laddon=((double)(wid-4-2*MaxW))*(addon)/(ar[1]-ar[0]),
      loc = ((double)(wid-4-2*MaxW))*(coord-ar[0])/(ar[1]-ar[0]);
    double xa = laddon, xi=loc, xl=xi-xa;
    //System.out.println("DBG: X-Axis >  c="+coord+" c+="+addon+" l="+loc+" l+="+laddon);
    for (; loc<=wid-4-2*MaxW; loc+=laddon, coord+=addon){
      String out = (xlog?((coord<0)?"10":(new Double((int)Math.pow(10,coord)).toString())):
		    //(((((Math.abs(addon))<1.0))?
		    (new Double(((double)Math.round(coord*10000))/10000).toString()));
		    //(new Double(coord).toString()));
      g.setColor(fgrnd);
      if((loc>=0)&&(loc<=wid-4-2*MaxW)){
	g.drawString(out,(int)loc-(FM.stringWidth(out)/2),heig-3*FH/2-1+(FH/3));
	g.drawString(out,(int)loc-(FM.stringWidth(out)/2),-FH/2-1+(FH/3));
	if((coord<0)&&(xlog)){
	  String ne = (new Integer((int)coord)).toString();
	  g.setFont(new Font(font.getName(),
			     font.getStyle(), (int)(5*font.getSize()/6)));
	  g.drawString(ne,(int)loc+(FM.stringWidth(ne)/2),
		       heig-3*FH/2-1);
	  g.drawString(ne,(int)loc+(FM.stringWidth(ne)/2),-FH/2-1);
	  g.setFont(font);
	}
	g.drawLine((int)loc,0, (int)loc,tick);
	g.drawLine((int)loc,heig-2*(FH+2), (int)loc,heig-2*(FH+2)-tick);
	/*for (int i=0;i<heig-2*(FH+2); i+=tick)
	  g.drawLine((int)loc,i,(int)loc,i);*/
      }
    }
    /********/
    addon = ((ar[3]-ar[2]<0)?-1:1)*(Math.pow(10,(int)(Log(Math.abs(ar[3]-ar[2])) -1)));
    if((((heig-4-2*FH)/FH)<((ar[3]-ar[2])/addon)))
      if(ylog)
	addon=addon/10;
      else{
     	boolean fiver = true;
	while(((heig-4-2*FH)/FH)<((ar[3]-ar[2])/addon)){
	  addon = addon*(fiver?5:2);
	  fiver = !fiver;
	}
      }
    coord=(((int)((ar[2]+addon)/addon))*addon);
    if(ylog)
      coord -=addon;
    laddon=((double)(heig-4-2*FH))*(addon)/(ar[3]-ar[2]);
    loc = ((double)(heig-4-2*FH))*(coord-ar[2])/(ar[3]-ar[2]);
    double ya = laddon, yi = loc, yl=yi-ya;
    //System.out.println("DBG: Y-Axis >  c="+coord+" c+="+addon+" l="+loc+" l+="+laddon);
    for (; loc<=heig-4-2*FH+laddon; loc+=laddon, coord+=addon){
      //String out = (new Double(coord)).toString();
      String out = (ylog?((coord<0)?"10":(new Double(Math.pow(10,coord)).toString())):
		    //(((((Math.abs(addon))<1.0))?
		    (new Double(((double)Math.round(coord*10000))/10000).toString()));
		    //(new Double(coord).toString()));
      g.setColor(fgrnd);
      if((loc>=0)&&(loc<=heig-4-2*FH)){
	g.drawString(out,(-MaxW-2-FM.stringWidth(out))/2,
		     (heig -2*FH - 4 -((int)loc-FH/3))-1);
	g.drawString(out,wid-2*MaxW-4+(MaxW+2-FM.stringWidth(out))/2,
		     (heig -2*FH -4 -((int)loc-FH/3))-1);
	if((coord<0)&&(ylog)){
	  String ne = (new Integer((int)coord)).toString();
	  g.setFont(new Font(font.getName(),
			     font.getStyle(), (int)(5*font.getSize()/6)));
	  g.drawString(ne,wid - MaxW-2-(MaxW+2)/2+FM.stringWidth(ne)/2,
		       heig-2*FH -4 -(int)loc);
	  g.drawString(ne,-(MaxW+2)/2+FM.stringWidth(ne)/2,
		       heig-2*FH -4 -(int)loc);
	  g.setFont(font);
	}
	g.drawLine(0, (int)(heig-4-2*FH-loc),tick,(int)(heig-4-2*FH-loc));
	g.drawLine(wid-2*(MaxW+2), (int)(heig-4-2*FH-loc),wid-2*(MaxW+2)-tick,
		   (int)(heig-4-2*FH-loc));
	/*for (int i=0;i<wid-2*(MaxW+2); i+=tick)
	  g.drawLine(i,heig-4-2*FH-(int)loc,i,heig-4-2*FH-(int)loc);*/
      }
    }
    for (xl=xi-xa;xl<wid-2*(MaxW+2)+xi+xa;xl+=xa)
      for (yl=yi-ya;yl<heig-2*(FH+2)+yi+ya; yl+=ya){
	if((xl>=0)&&(xl<=wid-2*(MaxW+2)) && (yl<=heig-2*(FH+2))&&(yl>=0))
	  //g.drawLine((int)xl,(int)(heig-4-2*FH-yl),(int)xl,(int)(heig-4-2*FH-yl));
	  drawTic(g,(int)xl,(int)(heig-4-2*FH-yl));
	if(xlog&&ylog){
	  for(double z=2,ch=Log(2/1); z<=10; z++,ch+=Log(z/(z-1)))
	    for(double n=2,cho=Log(2/1); n<=10; n++,cho+=Log(n/(n-1)))
	      if((heig-4-2*FH-(yl+ch*ya)<heig-4-2*FH)&&(heig-4-2*FH-(yl+ch*ya)>0)
		 &&(xl+cho*xa<wid-4-2*MaxW)&&(xl+cho*xa>0))
		//g.drawLine((int)(xl+cho*xa),(int)(heig-4-2*FH-(yl+ch*ya)),
		//   (int)(xl+cho*xa),(int)(heig-4-2*FH-(yl+ch*ya)));
		drawTic(g,(int)(xl+cho*xa),(int)(heig-4-2*FH-(yl+ch*ya)));
       	}else if(ylog){
	  for(double z=2,ch=Log(2/1); z<=10; z++,ch+=Log(z/(z-1)))
	    if((xl>=0)&&(xl<=wid-2*(MaxW+2)) && (heig-4-2*FH-(yl+ch*ya)<heig-4-2*FH)&&(heig-4-2*FH-(yl+ch*ya)>0))
	      // g.drawLine((int)xl,(int)(heig-4-2*FH-(yl+ch*ya)),(int)xl,
	      //	 (int)(heig-4-2*FH-(yl+ch*ya)));
	      drawTic(g,(int)xl,(int)(heig-4-2*FH-(yl+ch*ya)));
	}else if(xlog){
	  for(double z=2,ch=Log(2/1); z<10; z++,ch+=Log(z/(z-1)))
	    if((xl+ch*xa<wid-4-2*MaxW)&&(xl+ch*xa>0) && (yl<=heig-2*(FH+2))&&(yl>=0))
	      //g.drawLine((int)(xl+ch*xa),(int)(heig -2*FH-4-yl),
	      //	 (int)(xl+ch*xa), (int)(heig -2*FH-4-yl));
	      drawTic(g,(int)(xl+ch*xa), (int)(heig -2*FH-4-yl));
	}else{
	  if((yl-ya/2>0)&&(yl-ya/2<heig-2*(FH+2))&&(xl>0) && (xl<wid-2*(MaxW+2)))
	    //g.drawLine((int)(xl),(int)(heig-4-2*FH-yl+ya/2),(int)(xl),(int)(heig-4-2*FH-yl+ya/2));
	    drawTic(g,(int)(xl),(int)(heig-4-2*FH-yl+ya/2));
	  if((yl>0)&&(yl<heig-2*(FH+2))&&(xl-xa/2>0) && (xl-xa/2<wid-2*(MaxW+2)))
	    //g.drawLine((int)(xl-xa/2),(int)(heig-4-2*FH-yl),(int)(xl-xa/2),(int)(heig-4-2*FH-yl));
	    drawTic(g,(int)(xl-xa/2),(int)(heig-4-2*FH-yl));
	}
      }
    if((r.width!=0) && (r.height!=0))
      super.draw(gout);
    g.setColor(dark(getBackground()));
    g.drawLine(0,0,wid-4-2*MaxW,0);
    g.drawLine(0,0,0,heig-4-2*FH);
    g.setColor(light(getBackground()));
    g.drawLine(wid-4-2*MaxW,heig-4-2*FH,0,heig-4-2*FH);
    g.drawLine(wid-4-2*MaxW,heig-4-2*FH,wid-4-2*MaxW,0);
  }
  private void drawTic(Graphics g, int x, int y){
    //g.fillOval(x-1,y-1,3,3);
    int[] xc={x,x-1,x,x+1,x},yc={y-1,y,y+1,y,y-1};
    g.drawPolygon(xc,yc,5);
  }
  private int[]prevc={0,0};
  public boolean mouseMove(Event evt, int  x, int  y) {
    if (!inCanvas(x,y)){
      if(inCanvas(prevc[0],prevc[1]))
	super.mouseExit(evt,x,y);
      prevc[0]=x;
      prevc[1]=y;
      return true;
    }
    if(! inCanvas(prevc[0],prevc[1]))
      super.mouseEnter(evt,x,y);
    prevc[0]=x;
    prevc[1]=y;
    //System.out.println("DBG: Mouse = ( "+(x)+" , " + (y)+" )");
    return super.mouseMove(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseDrag(Event evt, int  x, int  y) {
    if (!inCanvas(x,y)){
      if(inCanvas(prevc[0],prevc[1]))
	super.mouseExit(evt,x,y);
      prevc[0]=x;
      prevc[1]=y;
      return true;
    }
    if(! inCanvas(prevc[0],prevc[1]))
      super.mouseEnter(evt,x,y);
    prevc[0]=x;
    prevc[1]=y;
    return super.mouseDrag(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseUp(Event evt, int  x, int  y) {
    if (!inCanvas(x,y))
      return true;
    //System.out.println("DBG: Mouse = ( "+(x)+" , " + (y)+" )");
    return super.mouseUp(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseDown(Event evt, int  x, int  y) {
    if (!inCanvas(x,y))
      return true;
    else
      return super.mouseDown(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseEnter(Event evt, int  x, int  y) 
    { 
    if (!inCanvas(x,y))
      return true;
    else
      return super.mouseEnter(evt,x-(MaxW+2), y-(FH+2));
    }
  public boolean mouseExit(Event evt, int  x, int  y) 
    { 
   if (!inCanvas(x,y))
      return true;
    else
      return super.mouseExit(evt,x-(MaxW+2), y-(FH+2));
     }
  private boolean inCanvas(int x,int y){
    return (!((x<MaxW+2) || (x>(wid-(MaxW+2))) || (y<FH+2) || (y>heig-(FH+2))));
  }
  public void setExtents(DataSource _in){
    if (_in != null)
      _in.deleteObserver(this);
    in=_in;
    if (_in != null){
      _in.addObserver(this);
      //ext = in.getExtents();
      redraw();
    }else
      ext = null;
  }
  public void repaint(int a,int b,int c,int d){
    if((c<wid-2*MaxW)&&(d<wid-2*FH))
      super.repaint(a+MaxW+2, b+FH+2, c,d);
    else
      super.repaint(a,b,c,d);
  }
  public void repaint(long t,int a,int b,int c,int d){
    if((c<wid-2*MaxW)&&(d<wid-2*FH))
      super.repaint(t, a+MaxW+2, b+FH+2, c,d);
    else
      super.repaint(t,a,b,c,d);
  }
  public void update( Observable obs, Object obj ){
    canvasreshape(inx,iny,wid,heig);
    super.update(obs,obj);
  }
  private double Log(double n){
    if (n>0)
      return (Math.log(n))/(Math.log(10));
    return 0;
  }
  /*  private double ln(double n){
    if (n <0)
      return (-Math.log(-n));
    else if (n>0)
      return (Math.log(n));
    else 
      return 0;
  }*/
}
