/**
 *
 * Thu Apr 02 00:00:00 1998 RuledCanvasView.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observer;
import java.util.Observable;
import java.util.Vector;
import java.util.Date;

class RuledCanvasView extends CanvasView implements LogCapable{
  protected int wid=100,heig=100;
  public int div=10;
  public synchronized void setXBase(double n){ xlog = n;}
  public synchronized void setYBase(double n){ ylog = n;}
  public synchronized void setZBase(double _n){ }
  public synchronized double getXBase(){ return xlog;}
  public synchronized double getYBase(){ return ylog;}
  //public synchronized double getZBase(){ return 1;}
  private RuledCanvasView loop = this;
  private double xlog = 1.0, ylog = 1.0;
  public Color fgrnd=Color.black, bgrnd=Color.lightGray;
  private Font font = new Font("Roman", Font.PLAIN, 10);
  private FontMetrics FM;
  protected int FH =-1, MaxW=-1, inx=0, iny=0;
  protected DataSource in=null;
  protected double[] ext=null;
  public boolean redraw(){
    if (FM == null){
      setVisible(true);
      if (getGraphics() != null){
	FM = getGraphics().getFontMetrics(font);
	FH = FM.getHeight();
      }
    }
    boolean bufff = false;
    ext = in.getExtents();
    if (FM != null){
      int n=(Math.max(FM.stringWidth(new Integer((int)ext[2]).toString()),
		     FM.stringWidth(new Integer((int)ext[3]).toString())));
      if (ylog != 1.0)
	n += FM.stringWidth(new Integer(22).toString());
      if(n!= loop.MaxW)
	bufff = true;
      MaxW = n;
      loop.MaxW = n;
      super.reshape(inx+MaxW+2,iny+FH+2,wid-4-2*MaxW,heig-4-2*FH);
      //System.out.println("font width = " + MaxW + " height = " +FH);
      //}
      return bufff;
    }
    return false;
  }
  protected boolean arraycomp (double[] a, double [] b, int idx){
    if (a.length != b.length)
      return false;
    if (a.length == idx)
      return true;
    if (a[idx] == b[idx])
      return (arraycomp(a,b,idx+1));
    return false;
  }
  public void setBounds( int x, int y, int _w, int _h) {reshape(x,y,_w,_h);}
  public void setBounds( Rectangle r){ reshape(r.x,r.y,r.width,r.height);}
  public void reshape( int x, int y, int _w, int _h ){
    inx = x;
    iny=y;
    wid = _w;
    heig = _h;
    redraw();
    canvasreshape(x,y,_w,_h);
  }
  public void resize(int w, int h){
    reshape(0,0,w,h);
  }
  public synchronized void draw(Graphics g){
    g.clipRect(1,1,wid,heig);
    g.setFont(font);
    g.setColor(fgrnd);
    if (FM == null){
      setVisible(true);
      if (g != null){
	FM = g.getFontMetrics();
	FH = FM.getHeight();
      }
    }
    double[]fr = loop.in.getExtents();
    if (FM != null){
      int n=(Math.max(FM.stringWidth(new Integer((int)fr[2]).toString()),
		      FM.stringWidth(new Integer((int)fr[3]).toString())));
      if (ylog != 1.0)
	n += FM.stringWidth(new Integer(22).toString());
      if ((n!=MaxW) || (fr[2] != ext[2]) || (fr[3] != ext[3]) || (fr[0] != ext[0]) || (fr[1] != ext[1])){
	//MaxW=n;
	loop.reshape(inx,iny,wid,heig);
	//redraw();
	//super.reshape(inx+MaxW+2,iny+FH+2,wid-4-2*MaxW,heig-4-2*FH); 
	return;
      }
    }
      //redraw();
    
    //Graphics gout = (g.create(MaxW+2,FH+2,wid-4-2*MaxW,heig-4-2*FH));
    //Rectangle r = g.getClipRect();//,q = g.getClipRect();
    //    int rx = r.x, ry = r.y;
    g.translate(MaxW+2, FH+2);
    //g.setClip(new Rectangle(rx+MaxW+2, ry+FH+2,r.width, r.height));
    Graphics gout = g.create(0,0,wid-4-2*MaxW, heig-4-2*FH);
    //g.clipRect(0,0,wid-4-2*MaxW, heig-4-2*FH);
    Rectangle r = gout.getClipRect();//,q = g.getClipRect();
    if((r.width!=0) && (r.height!=0))
      super.draw(gout);//.create(0,0,wid-4-2*MaxW, heig-4-2*FH));  
    for (int type = 0; type <=1; type++){
      double coord = baseLog(ext[2*type], ((type <1)? xlog:ylog)),
	addon=(baseLog(ext[2*type+1],((type<1)?xlog:ylog))-coord)/((double)div),
	loc = 0,//(type<1?MaxW:FH)+2,
	laddon = ((double)(type<1?(wid-4-2*MaxW):(heig-4-2*FH)))/((double)div);
      for (int i = 0; i<=div; i++, loc+=laddon, coord+=addon){
	String out = ((((Math.abs(addon))<1.0))?
		      (new Double(((double)Math.round(coord*100))/100).toString()):
		      (new Integer((int)coord).toString()));
	g.setColor(fgrnd);
	if (type <1){
	  if (xlog == 1.0){
	    g.drawString(out,(int)loc-(FM.stringWidth(out)/2),
			 heig-3*FH/2-1+(FH/3));
	    g.drawString(out,(int)loc-(FM.stringWidth(out)/2),
-FH/2-1+(FH/3));
	  } else {
	    String base = (new Double(((double)Math.round(xlog*100))/100).toString());
	    g.drawString(base,(int)loc-(FM.stringWidth(base)/2),
			 heig-3*FH/2-1+(FH/3));
	    g.drawString(base,(int)loc-(FM.stringWidth(base)/2),
-FH/2-1+(FH/3));
	    g.setFont(new Font(font.getName(),
			       font.getStyle(), (int)(5*font.getSize()/6)));
	    g.drawString(out,(int)loc+(FM.stringWidth(base)/2),
			 heig-3*FH/2-1);
	    g.drawString(out,(int)loc+(FM.stringWidth(base)/2),
-FH/2-1);
	    g.setFont(font);
	  }
	}else{
	  if (ylog == 1.0){
	    g.drawString(out,(-MaxW-2-FM.stringWidth(out))/2,
			 (heig -2*FH - 4 -((int)loc-FH/3))-1);
	    g.drawString(out,wid-2*MaxW-4+(MaxW+2-FM.stringWidth(out))/2,
			 (heig -2*FH -4 -((int)loc-FH/3))-1);
	  } else {
	    String base = (new Double(((double)Math.round(ylog*100))/100).toString());
	    g.drawString(base,-MaxW-1,
			 (heig -2*FH -4 -((int)loc-FH/3))-1);
	    g.drawString(base,wid-2*MaxW-3,
			 (heig -2*FH -4 -((int)loc-FH/3))-1);
	    g.setFont(new Font(font.getName(),
			       font.getStyle(), (int)(5*font.getSize()/6)));
	    g.drawString(out,wid - MaxW-2-(MaxW-1-FM.stringWidth(base)),
			 heig-2*FH -4 -(int)loc);
	    g.drawString(out,-MaxW-1+FM.stringWidth(base),
			 heig-2*FH -4 -(int)loc);
	    g.setFont(font);
	  }
	}
      }
    }
  }
  public boolean mouseMove(Event evt, int  x, int  y) {
    if ((x<MaxW+2) || (x>(wid-(MaxW+2))) || (y<FH+2) || (y>heig-(FH+2)))
      return true;
    //System.out.println("DBG: Mouse = ( "+(x)+" , " + (y)+" )");
    return super.mouseMove(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseDrag(Event evt, int  x, int  y) {
    if ((x<MaxW+2) || (x>(wid-(MaxW+2))) || (y<FH+2) || (y>heig-(FH+2)))
      return true;
    else
      return super.mouseDrag(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseUp(Event evt, int  x, int  y) {
    if ((x<MaxW+2) || (x>(wid-(MaxW+2))) || (y<FH+2) || (y>heig-(FH+2)))
      return true;
    //System.out.println("DBG: Mouse = ( "+(x)+" , " + (y)+" )");
    return super.mouseUp(evt,x-(MaxW+2), y-(FH+2));
  }
  public boolean mouseDown(Event evt, int  x, int  y) {
    if ((x<MaxW+2) || (x>(wid-(MaxW+2))) || (y<FH+2) || (y>heig-(FH+2)))
      return true;
    else
      return super.mouseDown(evt,x-(MaxW+2), y-(FH+2));
  }
  public void setExtents(DataSource _in){
    if (_in != null)
      _in.deleteObserver(this);
    in=_in;
    if (_in != null){
      _in.addObserver(this);
      //ext = in.getExtents();
      redraw();
    }else
      ext = null;
  }
  public void repaint(int a,int b,int c,int d){
    if((c<wid-2*MaxW)&&(d<wid-2*FH))
      super.repaint(a+MaxW+2, b+FH+2, c,d);
    else
      super.repaint(a,b,c,d);
  }
  public void repaint(long t,int a,int b,int c,int d){
    if((c<wid-2*MaxW)&&(d<wid-2*FH))
      super.repaint(t, a+MaxW+2, b+FH+2, c,d);
    else
      super.repaint(t,a,b,c,d);
  }
  public void update( Observable obs, Object obj ){
    canvasreshape(inx,iny,wid,heig);
    super.update(obs,obj);
  }
  private double baseLog(double n, double bas){
    if (bas == 1)
      return n;
    if ((bas > 0) && (n > 0))
      return (Math.log(n))/(Math.log(bas));
    else 
      return 0;
  }
}
