/**
 *
 * Fri Apr 24 01:00:00 1998 SNormExtentsFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Observable;
import java.util.Observer;
//-----------------------------------------
/** 
  * The S in SNormExtentsFilter  means Static.  This class is almost exactly like 
  * it's super class, NormExtentsFilter (Norm = Normalized), except for two
  * things: logorithms are supported and the base extents of the input object are
  * not kept.  That is to say
  * every time you open up a StaticNormalized dialog, it says the current
  * extents are 1 to -1, regardless of what had changed.  If another object
  * called getNormExtents from this object, this object would just send out a
  * copy of the NormalizedBaseExtents.  This class makes it easier to keep
  * multiple charts in spec and brings about simple zooming and panning
  * capabilities. a 50% zoom (-.5,.5) would be undone by a 200% (-2,2)
  * Also, by keeping the bases up, the zooming will be scale to viewing.  A linear
  * based zoom in a log based graph will only confuse people.  SNorm deals
  * more with teh actual view, the Norm deals more with the mathematical extents side.
  */

public class SNormExtentsFilter extends NormExtentsFilter implements LogCapable
{
  private boolean [] bases = {false,false,false};
  public void setXState(boolean n){bases[0]=(n);}
  public void setYState(boolean n){bases[1]=(n);}
  public void setZState(boolean n){bases[2]=(n);}

  public double[] getExtents(){
    //if ((in == null) || (in.getExtents() == null))
    //return super.getExtents();
    double[] output = super.getExtents();// new double[in.getExtents().length];
    //System.arraycopy(in.getExtents(), 0, output, 0, output.length);
    if(output!=null)
      for (int z=0; z<output.length/2; z++)
	if(bases[z])
	  if(output[2*z]<=0)
	    output[2*z]=0;
	  else 
	    if(output[2*z+1]<0)
	      output[2*z+1]=0;
    return output;
  }
  private boolean equal(double[] a, double[] b){
    if (a.length != b.length)
      return false;
    for (int x=0; x<a.length; x++)
      if (a[x] != b[x])
	return false;
    return true;
  }
  /**
    * called by the dialog
    *
    * @param _ext the new static normalized extents
    */
  public synchronized void dialogDone( double [] _ext) // called by dialog
    {  
      if (!(equal(new NormalizedBaseExtents().getExtents(),_ext))){
	Rescale(_ext);
	System.arraycopy(new NormalizedBaseExtents().getExtents(),0,_ext,0,_ext.length);
	setChanged(); 
	notifyObservers(); 
      }
    }
  /**
    * scales host object to normalized scale
    *
    * @param newext new normalized extents
    */
  protected synchronized void Rescale(double[] newext) 
    {
      if (newext != null){
	double [] repext =  (double[])new NormalizedBaseExtents().getExtents(), inext = in.getExtents();
	  double ol, tnl, tol;
	  for(int d=0; d<inext.length; d+=2){
	    if(bases[d/2])
	      ol = (log(inext[d+1])-log(inext[d]));
	    else
	      ol = (inext[d+1]-inext[d]);
	    tol = (repext[d+1]-repext[d]);
	    tnl = (newext[d+1]-newext[d]);
	    if ((tnl != 0) && (tol != 0)) {
	      if (tol != tnl){
		if (bases[d/2]){
		  inext[d] = e(((newext[d] - repext[d])/(tnl-tol))*(ol*(tnl/tol)-ol) + log(inext[d]));
		  inext[d+1] = e(((newext[d+1] - repext[d+1])/(tnl-tol))*(ol*(tnl/tol)-ol) + log(inext[d+1]));
		}else{
		  inext[d] = ((newext[d] - repext[d])/(tnl-tol))*(ol*(tnl/tol)-ol) + inext[d];
		  inext[d+1] = ((newext[d+1] - repext[d+1])/(tnl-tol))*(ol*(tnl/tol)-ol) + inext[d+1];
		}
	      } else {
		if(bases[d/2]){
		  inext[d] = e(((newext[d] - repext[d])*(ol/tol)) + log(inext[d]));
		  inext[d+1] = e(((newext[d+1] - repext[d+1])*(ol/tol)) + log(inext[d+1]));
		}else{
		  inext[d] = ((newext[d] - repext[d])*(ol/tol)) + inext[d];
		  inext[d+1] = ((newext[d+1] - repext[d+1])*(ol/tol)) + inext[d+1];
		}
	      }
	    }
	  }
      }
    }
  private double log(double n){
    if (n<=0)
      return -320.1;
    else
      return(Math.log(n));
  }
  private double e(double n){
    return (Math.pow(Math.E,n));
  }
  /**
    * resets its own extents to base extents.  this makes its extents Static
    */
   protected synchronized void elacseR() 
    {
      self =  new NormalizedBaseExtents();
    }
  /**
    * @return static normalized extents
    */
   public double[] getNormExtents()
     {
       elacseR();
       return self.getExtents();
     }
}
