/**
 *
 * Fri Mar 06 00:00:00 1998 SpikeFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.util.Observable;
import java.util.Observer;
/**
  * this class is kind of a kludge in how it works.  It takes out points
  * that have a difference of too much and smoothes them out.
  */
public class SpikeFilter extends DataFilter
{
  private int[] res={1,1,1};
  private double[] diff={100,100,100},cache=null,nu=null, dcache=null,dnu=null;
  private int dim;
  /**
    * sets input DataSource
    *
    * @param d the input DS
    */
  public synchronized void SetInput(DataSource d)
    {
      super.SetInput(d);
      if (in!=null){
	dim = in.getDimensions()[0];
	nu=in.getData();
	dnu=in.getExtents();
	//Fix();
      }
    }
  /**
    * Set the maximum difference to be allowed between two adjacent points in
    * the x data set
    *
    * @param _x the max x coord difference
    */
  public void SetXVariance(double _x)
    {if (_x>0) diff[0]=_x;}
  /**
    * set the width of the gap allowed in the x data set.  Example: the points
    * 1,999,999,999,1 would not be fixed by res 1 or 2, but 3 would catch it.
    * All smaller gaps are also caught.
    *
    * @param _x the new x resolution
    */
  public void SetXRes(int _x)
    {if (_x>0) res[0]=_x;}
  /**
    * Set the maximum difference to be allowed between two adjacent points in
    * the y data set
    *
    * @param _x the max y coord difference
    */
  public void SetYVariance(double _x)
    {if (_x>0) diff[1]=_x;}
  /**
    * set the width of the gap allowed in the y data set
    *
    * @param _x the new y resolution
    */
  public void SetYRes(int _x)
    {if (_x>0) res[1]=_x;}
  /**
    * Set the maximum difference to be allowed between two adjacent points in
    * the z data set
    *
    * @param _x the max z coord difference
    */
  public void SetZVariance(double _x)
    {if (_x>0) diff[2]=_x;}
  /**
    * set the width of the gap allowed in the z data set
    *
    * @param _x the new z resolution
    */
  public void SetZRes(int _x)
    {if (_x>0) res[2]=_x;}
  private synchronized void FixExtents()
    {
      for (int b=0; b<dim; b++){
	dnu[dim*b]=nu[b];
	dnu[dim*b+1]=nu[b];
      }
      for (int c=0; c<nu.length; c+=dim)
	for (int d=0; d<dim; d++){
	  dnu[dim*d] = Math.min(dnu[dim*d] ,nu[c+d]);
	  dnu[dim*d+1]=Math.max(dnu[dim*d+1],nu[c+d]);
	}
    }
  private synchronized void Filter(int start)
    {
      for (int b=3; b<=res[start]+2; b++)//resolution pan
	for (int c=start; c<(nu.length-b*dim+1); c+=dim)//array pan
	  if (!((nu[c]==Double.NaN) || (nu[c+dim]==Double.NaN)
		|| (nu[c+(b-1)*dim]==Double.NaN)))
	    if((Math.abs(nu[c]-nu[c+(b-1)*dim])<=((b-1)*diff[start])) 
	       && (Math.abs(nu[c]-nu[c+dim])>diff[start]))
	      for (int d=1; d<b-1;d++)//if jumps, pan between, remove.
		nu[c+(d*dim)]=nu[c]+((nu[c+(b-1)*dim]-nu[c])*d/(b-1));
    } 
  private synchronized void Fix()
    {
      for (int a=0; a<dim; a++)
	Filter(a);
      cache=nu;
      FixExtents();
      dcache=dnu;
    }

  public synchronized double[] getData()
    {
      if (cache!=null)
	return cache;
      Fix();
      return nu;
    }

  public synchronized double[] getExtents()
    {
      if (dcache!=null)
	return dcache;
      Fix();
      return dnu;
    }

  public void update( Observable it, Object data )
    {
      setChanged(); notifyObservers();
    } 
}
