/**
 *
 * Fri Mar 06 00:00:00 1998 URLTodoubleArrays.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.io.*;
import java.net.*;
import java.util.Vector;
/**
  * This class does ALL of its work upon instantiation.  Be prepared to live
  * with your connection/processor speed.
  */
public class URLTodoubleArrays
{
  /**
    * the URL to be dealt with
    */
  protected String filename = null;
  private char H_MARK = '~';
  private int dataLine = 0, headerLine = 0;
  private Vector header = new Vector(), units = new Vector();
  private boolean head = false, unit = false;
  /**
    * the matrix of doubles generated upon instantiation
    */
  protected double[][] data = null;
  private BufferedReader buff = null;
  private StreamTokenizer tk = null;
  /**
    * the URL generated from filename
    */
  protected URL conn= null;
  private InputStreamReader isr = null;
  private int colCount;
  /**
    * @param filen URL to be matricized
    * REMINDER: applets can only use files on the server of the CODEBASE.
    *           ignoring this will get a security error.
    */
  public URLTodoubleArrays(URL filen){
    conn = filen;
    initial();
  }
  /**
    * @param dl the line at which data begins at.  Default 1
    */
  public URLTodoubleArrays(URL filen, int dl){
    dataLine = dl;
    conn = filen;
    initial();
  }
  /**
    * @param hl the line with the labels in single words (no whitespace) default 0
    * @param label state of if the line after the label line contains the units of measure. default false
    */
  public URLTodoubleArrays(URL filen, int dl, int hl, boolean label){
    if (hl != 0){
      head = true;
      unit = label;
      headerLine = hl;
    }
    dataLine = dl;
    conn = filen;
    initial();
  }
  /**
    * @param cmnt a char to be considered a commented line by the StreamTokenizer.
    *
    * @see java.io.StreamTokenizer
    */
 public URLTodoubleArrays(URL filen,int dl,int hl,boolean label,char cmnt){
   if (hl != 0){
     head = true;
     unit = label;
     headerLine = hl;
   }
    H_MARK = cmnt;
    dataLine = dl;
    conn = filen;
    initial();
  }
  private synchronized void initial()
    {
      try{
	//SecurityManager sm = System.getSecurityManager();
	//sm.checkConnect("www.yahoo.com", 100);
	//System.out.println("DBG: URL " + conn.toString());
	//conn.openConnection();
	isr = new InputStreamReader(conn.openStream());
	buff = new BufferedReader(isr);
	tk = new StreamTokenizer(buff);
	//tk.whitespaceChars('a','z');
	//tk.whitespaceChars('A','Z');
	tk.quoteChar(H_MARK);
	tk.wordChars('/','/');
	tk.wordChars('%','%');
	tk.wordChars('$','$');
	tk.wordChars('#','#');
	tk.eolIsSignificant(true);
	tk.slashStarComments(true);
	tk.slashSlashComments(true);
	if (FindData()){
	System.out.println("Loading Data from URL '" + conn.toString() + "'.");
	  fillArrays();
	}else 
	System.out.println("No usable data in URL '" + conn.toString() + "'.");
	terminate();
      }
      catch(Exception fnf){
	System.out.println(fnf.toString() + ":  Returning Null.");
	terminate();
	return;
      }
    }
  private synchronized void terminate()
    {
      try{
	tk=null;
	buff.close();
	buff=null;
	isr.close();
	isr = null;
      }
      catch(IOException iox){
	System.out.println("LTDA::terminate> IOError.");
      }
    }
  private synchronized boolean FindData(){
    try{
      boolean chk;
      int line = 0;
      System.out.println("URL Opened.");
      if (head && (headerLine<dataLine)){
	for (;line<headerLine-1; line++)
	  buff.readLine();
	while(tk.nextToken() != tk.TT_EOL)
	  if (tk.ttype == tk.TT_WORD){
	    header.addElement(tk.sval);
	    //System.out.println("DBG: " + header.size() + ": " + tk.sval);
	  }
	if (unit){
	  while(tk.nextToken() != tk.TT_EOL)
	    if (tk.ttype == tk.TT_WORD){
	      units.addElement(tk.sval);
	      //System.out.println(tk.sval);
	    }
	  line++;
	}
	head = false;
      }
      if (line == dataLine-1) tk.pushBack();
      for (; line < dataLine-1; line++)
	buff.readLine();
      tk.nextToken();
      //System.out.println(tk.nval + " " + tk.sval);
      tk.pushBack();
      if (tk.ttype == tk.TT_EOF)
	return false;
      return true;
    }
    catch(IOException iox){
      System.out.println("LTDA::FindData> IOError.");
    }
    return false;
  }
  private synchronized void fixArrays(){
    try{
      Vector hold = new Vector();
      tk.nextToken();
      System.out.println("This should take about 1 second per 20k read.");
      while (tk.nextToken() != tk.TT_EOL){
	if (tk.ttype == tk.TT_NUMBER){
	  hold.addElement(new Double(tk.nval));
	  //System.out.println(tk.nval);
	}
      }
      colCount = hold.size();
      data = new double[hold.size()][1];
      //System.out.println(hold.size());
      for (int j=0; j<hold.size(); j++){
	//data[j]= new double[1];
	data[j][0]=((Double)hold.elementAt(j)).doubleValue();
      }
    }
    catch(IOException iox){
      System.out.println("LTDA::fixArrays> IOError.");
    }
  }
  private synchronized void fillArrays(){
    try{
      fixArrays();
      int col = 0, idx = 1;
      double[] hold = null;
      while (tk.nextToken() != tk.TT_EOF) {
	if (tk.ttype == tk.TT_EOL){
	  if (tk.nextToken() != tk.TT_NUMBER) {
	    if (tk.ttype == tk.TT_EOF)
	      break;
	    System.out.println("End of data reached.");
	    if(head){
	      tk.pushBack();
	      if(headerLine>tk.lineno())
		for (int line = tk.lineno();line<headerLine-1; line++)
		  buff.readLine();
	      while(tk.nextToken() != tk.TT_EOL)
		if (tk.ttype == tk.TT_WORD)
		  header.addElement(tk.sval);
	      if (unit)
		while(tk.nextToken() != tk.TT_EOL)
		  if (tk.ttype == tk.TT_WORD)
		    units.addElement(tk.sval);
	    }
	    break;
	  }
	  col = 0;
	  idx++;
	}
	if ((tk.ttype == tk.TT_NUMBER) && (col < data.length)){
	  hold = data[col];
	  data[col] = new double[idx + 1];
	  System.arraycopy(hold,0,data[col],0,idx);
	  data[col++][idx]=tk.nval;
	}
      }
      if (tk.ttype==tk.TT_EOF)
	System.out.println("End of file reached.");
    }
    catch(IOException iox){
      System.out.println("LTDA::fillArrays> IOError.");
    }
  }
  /** @see GetHeader
    */
  public synchronized String h(int x){
    return GetHeader(x);
  }
  /** @see GetUnits
    */
  public synchronized String u(int x){
    return GetUnits(x);
  }
  /** @see GetColumn
    */
  public synchronized double[] c(int x){
    return GetColumn(x);
  }
  /**
    * @return a string of the label for column x
    *
    * @param x  column number
    */
  public synchronized String GetHeader(int x){
    if (x <= header.size())
      return ((String)(header.elementAt(x-1)));
    System.out.println("No header for column " + x + ".");
    return ("no label");
  }
  /**
    * @return a string of the units for column x
    *
    * @param x   column number (min 1)
    */
  public synchronized String GetUnits(int x){
    if (x <= units.size())
      return ((String)(units.elementAt(x-1)));
    System.out.println("No unit for column " + x + ".");
    return ("-");
  }
  /**
    * @return a double array of the values in column x
    *
    * @param x   column number (min 1)
    */
  public synchronized double [] GetColumn(int x){
    try{
      return (double[])(data[x-1].clone());
    }
    catch(Exception ex){
      System.out.println("Column " + x + " does not exist as usable data in file '" + filename + "'.");
      return null;
    }
  }
  /**
    * @return the number of columns of data generated
    */
  public synchronized int ColumnCount(){
    return colCount;
  }
}
