/**
 *
 * Fri Feb 25 12:49:56 2000 ZeroLineAddingFilter.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/

import java.util.Observable;

public class ZeroLineAddingFilter extends DataFilter
{
	double [] cache = null; // cache only data
	boolean zerolining = true;
	
	public synchronized double[] getData()
	{
		if (cache!=null) return cache;
				
		if (in==null) return null;
		
		if (!zerolining) {
			double[] dat = in.getData();
			if (cacheEnabled()) cache = dat;
			return dat;
		}
		
		int[] dims = in.getDimensions();
		double[] ext = in.getExtents();
		double [] ol = in.getData();
		
		double [] nu = new double[ ol.length + dims[0] * 3 ];
		// prepend three tuples: opposite ends of a zero line and a NaN group
		
	// left line extent
		int i;
		for( i=0; i<dims[0]; i++ ) {
			nu[i] = (i==0)? ext[0] :
							(i==1)? 0.0 :
							Double.NaN;
		}
		
	// right line extent
		for( i=0; i<dims[0]; i++ ) {
			nu[i+dims[0]] = (i==0)? ext[1] :
							(i==1)? 0.0 :
							Double.NaN;
		}

	// spacer
		for( i=0; i<dims[0]; i++ ) {
			nu[i+2*dims[0]] = Double.NaN;
		}

	// copy the rest of the data
		int j=3*dims[0];
		for( i=0; i<ol.length; i++,j++ )
			nu[j] = ol[i];
		
		if (cacheEnabled()) cache = nu;
		return nu;
	}

	public boolean zeroLine() { return zerolining; }

	public synchronized void SetZeroLine( boolean st )
	{ 
		if (zerolining==st) return;
		zerolining = st;
		cache=null;
		setChanged(); notifyObservers();
	}

	public void update( Observable ob, Object pv )
	{ cache = null; setChanged(); notifyObservers(); }
	
}

